import os
import platform
import textwrap

import pytest

from conans.client.tools import replace_in_file
from conans.test.assets.cmake import gen_cmakelists
from conans.test.assets.genconanfile import GenConanfile
from conans.test.assets.pkg_cmake import pkg_cmake
from conans.test.assets.sources import gen_function_cpp, gen_function_h
from conans.test.utils.tools import TestClient


@pytest.fixture
def client():
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conans.tools import save
        import os
        class Pkg(ConanFile):
            settings = "build_type", "os", "arch", "compiler"
            {}
            def package(self):
                save(os.path.join(self.package_folder, "include", "%s.h" % self.name),
                     '#define MYVAR%s "%s"' % (self.name, self.settings.build_type))
        """)

    c.save({"conanfile.py": conanfile.format("")})
    c.run("create . liba/0.1@ -s build_type=Release")
    c.run("create . liba/0.1@ -s build_type=Debug")
    c.save({"conanfile.py": conanfile.format("requires = 'liba/0.1'")})
    c.run("create . libb/0.1@ -s build_type=Release")
    c.run("create . libb/0.1@ -s build_type=Debug")
    return c


@pytest.mark.tool_cmake
def test_transitive_multi(client):
    # TODO: Make a full linking example, with correct header transitivity

    # Save conanfile and example
    conanfile = textwrap.dedent("""
        [requires]
        libb/0.1

        [generators]
        CMakeDeps
        CMakeToolchain
        """)
    example_cpp = gen_function_cpp(name="main", includes=["libb", "liba"],
                                   preprocessor=["MYVARliba", "MYVARlibb"])
    client.save({"conanfile.txt": conanfile,
                 "CMakeLists.txt": gen_cmakelists(appname="example",
                                                  appsources=["example.cpp"], find_package=["libb"]),
                 "example.cpp": example_cpp}, clean_first=True)

    with client.chdir("build"):
        for bt in ("Debug", "Release"):
            client.run("install .. user/channel -s build_type={}".format(bt))

        # Test that we are using find_dependency with the NO_MODULE option
        # to skip finding first possible FindBye somewhere
        assert "find_dependency(${_DEPENDENCY} REQUIRED ${${_DEPENDENCY}_FIND_MODE})" \
               in client.load("libb-config.cmake")
        arch = client.get_default_host_profile().settings['arch']
        assert 'set(liba_FIND_MODE "NO_MODULE")' in client.load(f"libb-release-{arch}-data.cmake")

        if platform.system() == "Windows":
            client.run_command('cmake .. -DCMAKE_TOOLCHAIN_FILE=conan_toolchain.cmake')
            client.run_command('cmake --build . --config Debug')
            client.run_command('cmake --build . --config Release')

            client.run_command('Debug\\example.exe')
            assert "main: Debug!" in client.out
            assert "MYVARliba: Debug" in client.out
            assert "MYVARlibb: Debug" in client.out

            client.run_command('Release\\example.exe')
            assert "main: Release!" in client.out
            assert "MYVARliba: Release" in client.out
            assert "MYVARlibb: Release" in client.out
        else:
            # The CMakePresets IS MESSING WITH THE BUILD TYPE and then ignores the -D so I remove it
            replace_in_file(os.path.join(client.current_folder, "CMakePresets.json"),
                            "CMAKE_BUILD_TYPE", "DONT_MESS_WITH_BUILD_TYPE")
            for bt in ("Debug", "Release"):
                client.run_command('cmake .. -DCMAKE_BUILD_TYPE={} '
                                   '-DCMAKE_TOOLCHAIN_FILE=conan_toolchain.cmake'.format(bt))
                client.run_command('cmake --build . --clean-first')

                client.run_command('./example')
                assert "main: {}!".format(bt) in client.out
                assert "MYVARliba: {}".format(bt) in client.out
                assert "MYVARlibb: {}".format(bt) in client.out


@pytest.mark.tool_cmake
def test_system_libs():
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conans.tools import save
        import os

        class Test(ConanFile):
            name = "Test"
            version = "0.1"
            settings = "build_type"
            def package(self):
                save(os.path.join(self.package_folder, "lib/lib1.lib"), "")
                save(os.path.join(self.package_folder, "lib/liblib1.a"), "")

            def package_info(self):
                self.cpp_info.libs = ["lib1"]
                if self.settings.build_type == "Debug":
                    self.cpp_info.system_libs.append("sys1d")
                else:
                    self.cpp_info.system_libs.append("sys1")
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . -s build_type=Release")
    client.run("create . -s build_type=Debug")

    conanfile = textwrap.dedent("""
        [requires]
        Test/0.1

        [generators]
        CMakeDeps
        """)
    cmakelists = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(consumer NONE)
        set(CMAKE_PREFIX_PATH ${CMAKE_BINARY_DIR})
        set(CMAKE_MODULE_PATH ${CMAKE_BINARY_DIR})
        find_package(Test)
        message("System libs release: ${Test_SYSTEM_LIBS_RELEASE}")
        message("Libraries to Link release: ${Test_LIBS_RELEASE}")
        message("System libs debug: ${Test_SYSTEM_LIBS_DEBUG}")
        message("Libraries to Link debug: ${Test_LIBS_DEBUG}")
        get_target_property(tmp Test::Test INTERFACE_LINK_LIBRARIES)
        message("Target libs: ${tmp}")
        get_target_property(tmp CONAN_LIB::Test_lib1_%s INTERFACE_LINK_LIBRARIES)
        message("Micro-target libs: ${tmp}")
        get_target_property(tmp Test_DEPS_TARGET INTERFACE_LINK_LIBRARIES)
        message("Micro-target deps: ${tmp}")
        """)

    for build_type in ["Release", "Debug"]:
        client.save({"conanfile.txt": conanfile,
                     "CMakeLists.txt": cmakelists % build_type.upper()}, clean_first=True)
        client.run("install conanfile.txt -s build_type=%s" % build_type)
        client.run_command('cmake . -DCMAKE_BUILD_TYPE={0}'.format(build_type))

        library_name = "sys1d" if build_type == "Debug" else "sys1"
        # FIXME: Note it is CONAN_LIB::Test_lib1_RELEASE, not "lib1" as cmake_find_package
        if build_type == "Release":
            assert "System libs release: %s" % library_name in client.out
            assert "Libraries to Link release: lib1" in client.out
        else:
            assert "System libs debug: %s" % library_name in client.out
            assert "Libraries to Link debug: lib1" in client.out

        assert f"Target libs: $<$<CONFIG:{build_type}>:>;$<$<CONFIG:{build_type}>:CONAN_LIB::Test_lib1_{build_type.upper()}>" in client.out
        assert "Micro-target libs: Test_DEPS_TARGET" in client.out
        micro_target_deps = f"Micro-target deps: $<$<CONFIG:{build_type}>:>;$<$<CONFIG:{build_type}>:{library_name}>;" \
                            f"$<$<CONFIG:{build_type}>:>"
        assert micro_target_deps in client.out


@pytest.mark.tool_cmake
def test_system_libs_no_libs():
    """If the recipe doesn't declare cpp_info.libs then the target with the system deps, frameworks
       and transitive deps has to be linked to the global target"""
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conans.tools import save
        import os

        class Test(ConanFile):
            name = "Test"
            version = "0.1"
            settings = "build_type"

            def package_info(self):
                if self.settings.build_type == "Debug":
                    self.cpp_info.system_libs.append("sys1d")
                else:
                    self.cpp_info.system_libs.append("sys1")
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . -s build_type=Release")
    client.run("create . -s build_type=Debug")

    conanfile = textwrap.dedent("""
        [requires]
        Test/0.1

        [generators]
        CMakeDeps
        """)
    cmakelists = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(consumer NONE)
        set(CMAKE_PREFIX_PATH ${CMAKE_BINARY_DIR})
        set(CMAKE_MODULE_PATH ${CMAKE_BINARY_DIR})
        find_package(Test)
        message("System libs Release: ${Test_SYSTEM_LIBS_RELEASE}")
        message("Libraries to Link release: ${Test_LIBS_RELEASE}")
        message("System libs Debug: ${Test_SYSTEM_LIBS_DEBUG}")
        message("Libraries to Link debug: ${Test_LIBS_DEBUG}")
        get_target_property(tmp Test::Test INTERFACE_LINK_LIBRARIES)
        message("Target libs: ${tmp}")
        get_target_property(tmp Test_DEPS_TARGET INTERFACE_LINK_LIBRARIES)
        message("DEPS TARGET: ${tmp}")

        """)

    for build_type in ["Release", "Debug"]:
        client.save({"conanfile.txt": conanfile, "CMakeLists.txt": cmakelists}, clean_first=True)
        client.run("install conanfile.txt -s build_type=%s" % build_type)
        client.run_command('cmake . -DCMAKE_BUILD_TYPE={0}'.format(build_type))

        library_name = "sys1d" if build_type == "Debug" else "sys1"

        assert f"System libs {build_type}: {library_name}" in client.out
        assert f"Target libs: $<$<CONFIG:{build_type}>:>;$<$<CONFIG:{build_type}>:>;Test_DEPS_TARGET" in client.out
        assert f"DEPS TARGET: $<$<CONFIG:{build_type}>:>;" \
               f"$<$<CONFIG:{build_type}>:{library_name}>" in client.out


@pytest.mark.tool_cmake
def test_system_libs_components_no_libs():
    """If the recipe doesn't declare cpp_info.libs then the target with the system deps, frameworks
       and transitive deps has to be linked to the component target"""
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conans.tools import save
        import os

        class Test(ConanFile):
            name = "Test"
            version = "0.1"
            settings = "build_type"

            def package_info(self):
                if self.settings.build_type == "Debug":
                    self.cpp_info.components["foo"].system_libs.append("sys1d")
                else:
                    self.cpp_info.components["foo"].system_libs.append("sys1")
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . -s build_type=Release")
    client.run("create . -s build_type=Debug")

    conanfile = textwrap.dedent("""
        [requires]
        Test/0.1

        [generators]
        CMakeDeps
        """)
    cmakelists = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(consumer NONE)
        set(CMAKE_PREFIX_PATH ${CMAKE_BINARY_DIR})
        set(CMAKE_MODULE_PATH ${CMAKE_BINARY_DIR})
        find_package(Test)
        message("System libs Release: ${Test_Test_foo_SYSTEM_LIBS_RELEASE}")
        message("Libraries to Link release: ${Test_Test_foo_LIBS_RELEASE}")
        message("System libs Debug: ${Test_Test_foo_SYSTEM_LIBS_DEBUG}")
        message("Libraries to Link debug: ${Test_Test_foo_LIBS_DEBUG}")

        get_target_property(tmp Test::foo INTERFACE_LINK_LIBRARIES)
        message("Target libs: ${tmp}")
        get_target_property(tmp Test_Test_foo_DEPS_TARGET INTERFACE_LINK_LIBRARIES)
        message("DEPS TARGET: ${tmp}")

        """)

    for build_type in ["Release", "Debug"]:
        client.save({"conanfile.txt": conanfile, "CMakeLists.txt": cmakelists}, clean_first=True)
        client.run("install conanfile.txt -s build_type=%s" % build_type)
        client.run_command('cmake . -DCMAKE_BUILD_TYPE={0}'.format(build_type))

        library_name = "sys1d" if build_type == "Debug" else "sys1"

        assert f"System libs {build_type}: {library_name}" in client.out
        assert f"Target libs: $<$<CONFIG:{build_type}>:>;$<$<CONFIG:{build_type}>:>;Test_Test_foo_DEPS_TARGET" in client.out
        assert f"DEPS TARGET: $<$<CONFIG:{build_type}>:>;" \
               f"$<$<CONFIG:{build_type}>:{library_name}>" in client.out


@pytest.mark.tool_cmake
def test_do_not_mix_cflags_cxxflags():
    # TODO: Verify with components too
    client = TestClient()
    cpp_info = {"cflags": ["one", "two"], "cxxflags": ["three", "four"]}
    client.save({"conanfile.py": GenConanfile("upstream", "1.0").with_package_info(cpp_info=cpp_info,
                                                                                   env_info={})})
    client.run("create .")

    consumer_conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import CMake

        class Consumer(ConanFile):
            name = "consumer"
            version = "1.0"
            settings = "os", "compiler", "arch", "build_type"
            exports_sources = "CMakeLists.txt"
            requires = "upstream/1.0"
            generators = "CMakeDeps", "CMakeToolchain"

            def build(self):
                cmake = CMake(self)
                cmake.configure()
        """)
    cmakelists = textwrap.dedent("""
       cmake_minimum_required(VERSION 3.15)
       project(consumer NONE)
       find_package(upstream CONFIG REQUIRED)
       get_target_property(tmp upstream::upstream INTERFACE_COMPILE_OPTIONS)
       message("compile options: ${tmp}")
       message("cflags: ${upstream_COMPILE_OPTIONS_C_RELEASE}")
       message("cxxflags: ${upstream_COMPILE_OPTIONS_CXX_RELEASE}")
       """)
    client.save({"conanfile.py": consumer_conanfile,
                 "CMakeLists.txt": cmakelists}, clean_first=True)
    client.run("create .")
    assert "compile options: $<$<CONFIG:Release>:" \
           "$<$<COMPILE_LANGUAGE:CXX>:three;four>;$<$<COMPILE_LANGUAGE:C>:one;two>>" in client.out
    assert "cflags: one;two" in client.out
    assert "cxxflags: three;four" in client.out


def test_custom_configuration(client):
    """  The configuration may differ from the build context and the host context"""
    conanfile = textwrap.dedent("""
       from conans import ConanFile
       from conan.tools.cmake import CMakeDeps

       class Consumer(ConanFile):
           name = "consumer"
           version = "1.0"
           settings = "os", "compiler", "arch", "build_type"
           requires = "liba/0.1"
           build_requires = "liba/0.1"
           generators = "CMakeToolchain"

           def generate(self):
               cmake = CMakeDeps(self)
               cmake.configuration = "Debug"
               cmake.build_context_activated = ["liba"]
               cmake.build_context_suffix["liba"] = "_build"
               cmake.generate()
       """)
    host_arch = client.get_default_host_profile().settings['arch']
    client.save({"conanfile.py": conanfile})
    client.run("install . -pr:h default -s:b build_type=RelWithDebInfo"
               " -pr:b default -s:b arch=x86 --build missing")
    curdir = client.current_folder
    data_name_context_build = "liba_build-relwithdebinfo-x86-data.cmake"
    data_name_context_host = f"liba-debug-{host_arch}-data.cmake"
    assert os.path.exists(os.path.join(curdir, data_name_context_build))
    assert os.path.exists(os.path.join(curdir, data_name_context_host))

    assert "set(liba_build_INCLUDE_DIRS_RELWITHDEBINFO" in \
           open(os.path.join(curdir, data_name_context_build)).read()
    assert "set(liba_INCLUDE_DIRS_DEBUG" in \
           open(os.path.join(curdir, data_name_context_host)).read()


def test_buildirs_working():
    """  If a recipe declares cppinfo.buildirs those dirs will be exposed to be consumer
    to allow a cmake "include" function call after a find_package"""
    c = TestClient()
    conanfile = str(GenConanfile().with_name("my_lib").with_version("1.0")
                                  .with_import("import os").with_import("from conans import tools"))
    conanfile += """
    def package(self):
        tools.save(os.path.join(self.package_folder, "my_build_dir", "my_cmake_script.cmake"),
                   'set(MYVAR "Like a Rolling Stone")')

    def package_info(self):
        self.cpp_info.builddirs=["my_build_dir"]
    """

    c.save({"conanfile.py": conanfile})
    c.run("create .")

    consumer_conanfile = GenConanfile().with_name("consumer").with_version("1.0")\
        .with_cmake_build().with_require("my_lib/1.0") \
        .with_settings("os", "arch", "build_type", "compiler") \
        .with_exports_sources("*.txt")
    cmake = gen_cmakelists(find_package=["my_lib"])
    cmake += """
    message("CMAKE_MODULE_PATH: ${CMAKE_MODULE_PATH}")
    include("my_cmake_script")
    message("MYVAR=>${MYVAR}")
    """
    c.save({"conanfile.py": consumer_conanfile, "CMakeLists.txt": cmake})
    c.run("create .")
    assert "MYVAR=>Like a Rolling Stone" in c.out


@pytest.mark.tool_cmake
def test_cpp_info_link_objects():
    client = TestClient()
    obj_ext = "obj" if platform.system() == "Windows" else "o"
    cpp_info = {"objects": [os.path.join("lib", "myobject.{}".format(obj_ext))]}
    object_cpp = gen_function_cpp(name="myobject")
    object_h = gen_function_h(name="myobject")
    cmakelists = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(MyObject)
        file(GLOB HEADERS *.h)
        add_library(myobject OBJECT myobject.cpp)
        if( WIN32 )
            set(OBJ_PATH "myobject.dir/Release/myobject${CMAKE_C_OUTPUT_EXTENSION}")
        else()
            set(OBJ_PATH "CMakeFiles/myobject.dir/myobject.cpp${CMAKE_C_OUTPUT_EXTENSION}")
        endif()
        install(FILES ${CMAKE_CURRENT_BINARY_DIR}/${OBJ_PATH}
                DESTINATION ${CMAKE_INSTALL_PREFIX}/lib
                RENAME myobject${CMAKE_C_OUTPUT_EXTENSION})
        install(FILES ${HEADERS}
                DESTINATION ${CMAKE_INSTALL_PREFIX}/include)
    """)

    test_package_cpp = gen_function_cpp(name="main", includes=["myobject"], calls=["myobject"])
    test_package_cmakelists = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(example)
        find_package(myobject REQUIRED)
        add_executable(example example.cpp)
        target_link_libraries(example myobject::myobject)
    """)

    client.save({"CMakeLists.txt": cmakelists,
                 "conanfile.py": GenConanfile("myobject", "1.0").with_package_info(cpp_info=cpp_info,
                                                                                   env_info={})
                                                                .with_exports_sources("*")
                                                                .with_cmake_build()
                                                                .with_package("cmake = CMake(self)",
                                                                              "cmake.install()"),
                 "myobject.cpp": object_cpp,
                 "myobject.h": object_h,
                 "test_package/conanfile.py": GenConanfile().with_cmake_build()
                                                            .with_import("import os")
                                                            .with_test('path = "{}".format(self.settings.build_type) '
                                                                       'if self.settings.os == "Windows" else "."')
                                                            .with_test('self.run("{}{}example".format(path, os.sep))'),
                 "test_package/example.cpp": test_package_cpp,
                 "test_package/CMakeLists.txt": test_package_cmakelists})

    client.run("create . -s build_type=Release")
    assert "myobject: Release!" in client.out


def test_private_transitive():
    # https://github.com/conan-io/conan/issues/9514
    client = TestClient()
    client.save({"dep/conanfile.py": GenConanfile(),
                 "pkg/conanfile.py": GenConanfile().with_require("dep/0.1", private=True),
                 "consumer/conanfile.py": GenConanfile().with_requires("pkg/0.1")
                                                        .with_settings("os", "build_type", "arch")})
    client.run("create dep dep/0.1@")
    client.run("create pkg pkg/0.1@")
    client.run("install consumer -g CMakeDeps -s arch=x86_64 -s build_type=Release")
    assert "dep/0.1:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Skip" in client.out
    data_cmake = client.load("pkg-release-x86_64-data.cmake")
    assert 'set(pkg_FIND_DEPENDENCY_NAMES "")' in data_cmake


@pytest.mark.tool_cmake
def test_system_dep():
    """This test creates a zlib package and use the installation CMake FindZLIB.cmake to locate
    the library of the package. That happens because:
    - The package declares: self.cpp_info.set_property("cmake_find_mode", "none") so CMakeDeps does nothing
    - The toolchain set the CMAKE_LIBRARY_PATH to the "lib" of the package, so the library file is found
    """
    client = TestClient()
    files = pkg_cmake("zlib", "0.1")
    files["conanfile.py"] += """
    def package_info(self):
        # This will use the FindZLIB from CMake but will find this library package
        self.cpp_info.set_property("cmake_file_name", "ZLIB")
        self.cpp_info.set_property("cmake_target_name", "ZLIB::ZLIB")
        self.cpp_info.set_property("cmake_find_mode", "none")
    """
    client.save({os.path.join("zlib", name): content for name, content in files.items()})
    files = pkg_cmake("mylib", "0.1", requires=["zlib/0.1"])
    files["CMakeLists.txt"] = files["CMakeLists.txt"].replace("find_package(zlib)",
                                                              "find_package(ZLIB)")
    files["CMakeLists.txt"] = files["CMakeLists.txt"].replace("zlib::zlib","ZLIB::ZLIB")
    client.save({os.path.join("mylib", name): content for name, content in files.items()})
    files = pkg_cmake("consumer", "0.1", requires=["mylib/0.1"])
    client.save({os.path.join("consumer", name): content for name, content in files.items()})
    client.run("create zlib")
    client.run("create mylib")
    client.run("create consumer")
    assert "Found ZLIB:" in client.out

    client.run("install consumer")
    if platform.system() != "Windows":
        host_arch = client.get_default_host_profile().settings['arch']
        data = os.path.join(f"consumer/build/generators/mylib-release-{host_arch}-data.cmake")
        contents = client.load(data)
        assert 'set(ZLIB_FIND_MODE "")' in contents


@pytest.mark.tool_cmake(version="3.19")
def test_error_missing_build_type():
    # https://github.com/conan-io/conan/issues/11168
    client = TestClient()

    client.run("new hello/1.0 -m=cmake_lib")
    client.run("create . -tf=None")

    conanfile = textwrap.dedent("""
        [requires]
        hello/1.0
        [generators]
        CMakeDeps
        CMakeToolchain
    """)

    main = textwrap.dedent("""
        #include <hello.h>
        int main() {hello();return 0;}
    """)

    cmakelists = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(app)
        find_package(hello REQUIRED)
        add_executable(app)
        target_link_libraries(app hello::hello)
        target_sources(app PRIVATE main.cpp)
    """)

    if platform.system() != "Windows":
        client.save({
            "conanfile.txt": conanfile,
            "main.cpp": main,
            "CMakeLists.txt": cmakelists
        }, clean_first=True)

        client.run("install .")
        client.run_command("cmake . -DCMAKE_TOOLCHAIN_FILE=conan_toolchain.cmake -G 'Unix Makefiles'", assert_error=True)
        assert "Please, set the CMAKE_BUILD_TYPE variable when calling to CMake" in client.out

    client.save({
        "conanfile.txt": conanfile,
        "main.cpp": main,
        "CMakeLists.txt": cmakelists
    }, clean_first=True)

    client.run("install .")

    generator = {
        "Windows": '-G "Visual Studio 15 2017"',
        "Darwin": '-G "Xcode"',
        "Linux": '-G "Ninja Multi-Config"'
    }.get(platform.system())

    client.run_command("cmake . -DCMAKE_TOOLCHAIN_FILE=conan_toolchain.cmake {}".format(generator))
    client.run_command("cmake --build . --config Release")
    run_app = r".\Release\app.exe" if platform.system() == "Windows" else "./Release/app"
    client.run_command(run_app)
    assert "Hello World Release!" in client.out



import os
import textwrap

from conans.test.utils.scm import create_local_git_repo
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import TestClient
from conans.util.files import mkdir


def test_third_party_patch_flow():
    """ this test emulates the work of a developer contributing recipes to ConanCenter, and having
    to do multiple patches to the original library source code:
    - Everything is local, not messing with the cache
    - Using layout() to define location of things
    """
    conanfile = textwrap.dedent(r"""
        import os
        from conan import ConanFile
        from conan.tools.files import save, load, apply_conandata_patches

        class Pkg(ConanFile):
            name = "mypkg"
            version = "1.0"
            exports_sources = "*"

            def layout(self):
                self.folders.source = "src"
                self.folders.build = "build"

            def source(self):
                # emulate a download from web site
                save(self, "myfile.cpp", "mistake1\nsomething\nmistake2\nmistake3\nsome\n")
                apply_conandata_patches(self)

            def build(self):
                content = load(self,  os.path.join(self.source_folder, "myfile.cpp"))
                for i in (1, 2, 3):
                    if "mistake{}".format(i) in content:
                        raise Exception("MISTAKE{} BUILD!".format(i))
        """)

    client = TestClient()
    client.save({"conanfile.py": conanfile,
                 "conandata.yml": ""})
    client.run("install .")
    client.run("source .")
    assert "apply_conandata_patches(): No patches defined in conandata" in client.out

    client.save({"conandata.yml": "patches: {}"})
    client.run("source .")
    client.run("build .", assert_error=True)
    assert "MISTAKE1 BUILD!" in client.out

    # user decides to create patches, first init the repo
    client.init_git_repo(folder="src")  # Using helper for user/email repo init
    client.save({"src/myfile.cpp": "correct1\nsomething\nmistake2\nmistake3\nsome\n"})
    # compute the patch
    mkdir(os.path.join(client.current_folder, "patches"))
    client.run_command("cd src && git diff > ../patches/patch1")
    client.run_command("cd src && git add . && git commit -m patch1")
    conandata = textwrap.dedent("""
        patches:
          "1.0":
            - patch_file: "patches/patch1"
        """)
    client.save({"conandata.yml": conandata})

    client.run("source .")
    client.run("build .", assert_error=True)
    assert "MISTAKE2 BUILD!" in client.out

    client.save({"src/myfile.cpp": "correct1\nsomething\ncorrect2\nmistake3\nsome\n"})
    # compute the patch
    mkdir(os.path.join(client.current_folder, "patches"))
    client.run_command("cd src && git diff > ../patches/patch2")
    client.run_command("cd src && git add . && git commit -m patch1")

    conandata = textwrap.dedent("""
        patches:
          "1.0":
            - patch_file: "patches/patch1"
            - patch_file: "patches/patch2"
        """)
    client.save({"conandata.yml": conandata})
    client.run("source .")
    client.run("build .", assert_error=True)
    assert "MISTAKE3 BUILD!" in client.out

    client.save({"src/myfile.cpp": "correct1\nsomething\ncorrect2\ncorrect3\nsome\n"})
    # compute the patch
    mkdir(os.path.join(client.current_folder, "patches"))
    client.run_command("cd src && git diff > ../patches/patch3")
    client.run_command("cd src && git add . && git commit -m patch1")

    conandata = textwrap.dedent("""
           patches:
             "1.0":
               - patch_file: "patches/patch1"
               - patch_file: "patches/patch2"
               - patch_file: "patches/patch3"
           """)
    client.save({"conandata.yml": conandata})
    client.run("source .")
    client.run("build .")
    assert "conanfile.py (mypkg/1.0): Calling build()" in client.out

    # of course create should work too
    client.run("create .")
    assert "mypkg/1.0: Created package" in client.out


def test_third_party_overwrite_build_file():
    """ this test emulates the work of a developer contributing recipes to ConanCenter, and
    replacing the original build script with your one one.

    The export_sources is actually copying CMakeLists.txt into the "src" folder, but the
    'download' will overwrite it, so it is necessary to copy it again
    """
    conanfile = textwrap.dedent(r"""
        import os, shutil
        from conan import ConanFile
        from conan.tools.files import save, load

        class Pkg(ConanFile):
            name = "mypkg"
            version = "1.0"
            exports_sources = "CMakeLists.txt"

            def layout(self):
                self.folders.source = "src"
                self.folders.build = "build"

            def source(self):
                # emulate a download from web site
                save(self, "CMakeLists.txt", "MISTAKE: Very old CMakeLists to be replaced")
                # Now I fix it with one of the exported files
                shutil.copy("../CMakeLists.txt", ".")

            def build(self):
                if "MISTAKE" in load(self, os.path.join(self.source_folder, "CMakeLists.txt")):
                    raise Exception("MISTAKE BUILD!")
        """)

    client = TestClient()
    client.save({"conanfile.py": conanfile,
                 "conandata.yml": "",
                 "CMakeLists.txt": "My better cmake"})
    client.run("install .")
    client.run("source .")
    client.run("build .")
    assert "conanfile.py (mypkg/1.0): Calling build()" in client.out

    # of course create should work too
    client.run("create .")
    assert "mypkg/1.0: Created package" in client.out


def test_third_party_git_overwrite_build_file():
    """ Same as the above, but using git clone
    The trick: "git clone <url> ." needs an empty directory. No reason why the ``src`` folder should
    be polluted automatically with exports, so just removing things works
    """
    git_repo = temp_folder().replace("\\", "/")
    create_local_git_repo({"CMakeLists.txt": "MISTAKE Cmake"}, folder=git_repo)

    conanfile = textwrap.dedent(r"""
        import os, shutil
        from conan import ConanFile
        from conan.tools.files import save, load

        class Pkg(ConanFile):
            name = "mypkg"
            version = "1.0"
            exports_sources = "CMakeLists.txt"

            def layout(self):
                self.folders.source = "src"
                self.folders.build = "build"

            def source(self):
                self.output.info("CWD: {{}}!".format(os.getcwd()))
                self.output.info("FILES: {{}}!".format(sorted(os.listdir("."))))
                self.run('git clone "{}" .')
                # Now I fix it with one of the exported files
                shutil.copy("../CMakeLists.txt", ".")

            def build(self):
                if "MISTAKE" in load(self, os.path.join(self.source_folder, "CMakeLists.txt")):
                    raise Exception("MISTAKE BUILD!")
        """.format(git_repo))

    client = TestClient()
    client.save({"conanfile.py": conanfile,
                 "conandata.yml": "",
                 "CMakeLists.txt": "My better cmake"})
    client.run("install .")
    client.run("source .")
    assert "FILES: []!" in client.out
    client.run("build .")
    assert "conanfile.py (mypkg/1.0): Calling build()" in client.out

    # of course create should work too
    client.run("create .")
    assert "mypkg/1.0: Created package" in client.out



import mock
import unittest

from mock import Mock
from parameterized import parameterized

from conans.client import tools
from conans.client.conf.detect import detect_defaults_settings
from conans.paths import DEFAULT_PROFILE_NAME
from conans.test.utils.mocks import TestBufferConanOutput


class DetectTest(unittest.TestCase):
    @mock.patch("platform.machine", return_value="")
    def test_detect_empty_arch(self, _):
        result = detect_defaults_settings(output=Mock(),
                                          profile_path=DEFAULT_PROFILE_NAME)
        result = dict(result)
        self.assertTrue("arch" not in result)
        self.assertTrue("arch_build" not in result)

    @mock.patch("conans.client.conf.detect._gcc_compiler", return_value=("gcc", "8"))
    def test_detect_custom_profile(self, _):
        output = TestBufferConanOutput()
        with tools.environment_append({"CC": "gcc"}):
            detect_defaults_settings(output, profile_path="~/.conan/profiles/mycustomprofile")
            self.assertIn("conan profile update settings.compiler.libcxx=libstdc++11 "
                          "mycustomprofile", output)

    @mock.patch("conans.client.conf.detect._gcc_compiler", return_value=("gcc", "8"))
    def test_detect_default_profile(self, _):
        output = TestBufferConanOutput()
        with tools.environment_append({"CC": "gcc"}):
            detect_defaults_settings(output, profile_path="~/.conan/profiles/default")
            self.assertIn("conan profile update settings.compiler.libcxx=libstdc++11 default",
                          output)

    @mock.patch("conans.client.conf.detect._gcc_compiler", return_value=("gcc", "8"))
    def test_detect_file_profile(self, _):
        output = TestBufferConanOutput()
        with tools.environment_append({"CC": "gcc"}):
            detect_defaults_settings(output, profile_path="./MyProfile")
            self.assertIn("conan profile update settings.compiler.libcxx=libstdc++11 MyProfile",
                          output)

    @mock.patch("conans.client.conf.detect._gcc_compiler", return_value=("gcc", "8"))
    def test_detect_abs_file_profile(self, _):
        output = TestBufferConanOutput()
        with tools.environment_append({"CC": "gcc"}):
            detect_defaults_settings(output, profile_path="/foo/bar/quz/custom-profile")
            self.assertIn("conan profile update settings.compiler.libcxx=libstdc++11 "
                          "custom-profile", output)

    @parameterized.expand([
        ['powerpc', '64', '7.1.0.0', 'ppc64'],
        ['powerpc', '32', '7.1.0.0', 'ppc32'],
        ['rs6000', None, '4.2.1.0', 'ppc32']
    ])
    def test_detect_aix(self, processor, bitness, version, expected_arch):
        with mock.patch("platform.machine", mock.MagicMock(return_value='XXXXXXXXXXXX')), \
                mock.patch("platform.processor", mock.MagicMock(return_value=processor)), \
                mock.patch("platform.system", mock.MagicMock(return_value='AIX')), \
                mock.patch("conans.client.tools.oss.OSInfo.get_aix_conf", mock.MagicMock(return_value=bitness)), \
                mock.patch('subprocess.check_output', mock.MagicMock(return_value=version)):
            result = detect_defaults_settings(output=Mock(),
                                              profile_path=DEFAULT_PROFILE_NAME)
            result = dict(result)
            self.assertEqual("AIX", result['os'])
            self.assertEqual("AIX", result['os_build'])
            self.assertEqual(expected_arch, result['arch'])
            self.assertEqual(expected_arch, result['arch_build'])

    @parameterized.expand([
        ['arm64', 'armv8'],
        ['i386', 'x86'],
        ['i686', 'x86'],
        ['i86pc', 'x86'],
        ['amd64', 'x86_64'],
        ['aarch64', 'armv8'],
        ['sun4v', 'sparc']
    ])
    def test_detect_arch(self, machine, expected_arch):
        with mock.patch("platform.machine", mock.MagicMock(return_value=machine)):
            result = detect_defaults_settings(output=Mock(),
                                              profile_path=DEFAULT_PROFILE_NAME)
            result = dict(result)
            self.assertEqual(expected_arch, result['arch'])
            self.assertEqual(expected_arch, result['arch_build'])

    @mock.patch("conans.client.conf.detect._clang_compiler", return_value=("clang", "9"))
    def test_detect_clang_gcc_toolchain(self, _):
        output = TestBufferConanOutput()
        with tools.environment_append({"CC": "clang-9 --gcc-toolchain=/usr/lib/gcc/x86_64-linux-gnu/9"}):
            detect_defaults_settings(output, profile_path="./MyProfile")
            self.assertIn("CC and CXX: clang-9 --gcc-toolchain", output)

    def test_vs2022(self):
        with mock.patch("conans.client.conf.detect._get_default_compiler",
                        mock.MagicMock(return_value=("Visual Studio", "17"))):
            result = detect_defaults_settings(output=Mock(),
                                              profile_path=DEFAULT_PROFILE_NAME)
            result = dict(result)
            self.assertEqual('Visual Studio', result['compiler'])
            self.assertEqual('17', result['compiler.version'])



import textwrap
import unittest

import pytest

from conans.model.ref import ConanFileReference
from conans.paths import CONANFILE, CONANFILE_TXT
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient, TestServer

generator = """
from conans.model import Generator
from conans.paths import BUILD_INFO
from conans import ConanFile

class MyCustom_Generator(Generator):
    @property
    def filename(self):
        return "customfile.gen"

    @property
    def content(self):
        return "My custom generator content"


class MyCustomGeneratorPackage(ConanFile):
    name = "MyCustomGen"
    version = "0.2"
"""

consumer = """
[requires]
Hello0/0.1@lasote/stable
MyCustomGen/0.2@lasote/stable

[generators]
MyCustom_Generator
"""

generator_multi = """
from conans.model import Generator
from conans.paths import BUILD_INFO
from conans import ConanFile

class MyCustomMultiGenerator(Generator):
    @property
    def filename(self):
        return "customfile.gen"

    @property
    def content(self):
        return {"file1.gen": "CustomContent1",
                "file2.gen": "CustomContent2"}


class NoMatterTheName(ConanFile):
    name = "MyCustomGen"
    version = "0.2"
"""

consumer_multi = """
[requires]
MyCustomGen/0.2@lasote/stable

[generators]
MyCustomMultiGenerator
"""


class CustomGeneratorTest(unittest.TestCase):

    def setUp(self):
        test_server = TestServer()
        self.servers = {"default": test_server}

    def test_reuse(self):
        ref = ConanFileReference.loads("Hello0/0.1@lasote/stable")

        client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]})
        client.save({"conanfile.py": GenConanfile("Hello0", "0.1")})
        client.run("export . lasote/stable")
        client.run("upload %s" % str(ref))

        gen_ref = ConanFileReference.loads("MyCustomGen/0.2@lasote/stable")
        files = {CONANFILE: generator}
        client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]})
        client.save(files)
        client.run("export . lasote/stable")
        client.run("upload %s" % str(gen_ref))

        # Test local, no retrieval
        files = {CONANFILE_TXT: consumer}
        client.save(files, clean_first=True)
        client.run("install . --build")
        generated = client.load("customfile.gen")
        self.assertEqual(generated, "My custom generator content")

        # Test retrieval from remote
        client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]})
        files = {CONANFILE_TXT: consumer}
        client.save(files)
        client.run("install . --build")

        generated = client.load("customfile.gen")
        self.assertEqual(generated, "My custom generator content")

    def test_multifile(self):
        gen_ref = ConanFileReference.loads("MyCustomGen/0.2@lasote/stable")
        client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]})
        files = {CONANFILE: generator_multi}
        client.save(files)
        client.run("export . lasote/stable")
        client.run("upload %s" % str(gen_ref))

        # Test local, no retrieval
        files = {CONANFILE_TXT: consumer_multi}
        client.save(files, clean_first=True)
        client.run("install . --build")
        self.assertIn("Generator MyCustomMultiGenerator is multifile. "
                      "Property 'filename' not used",
                      client.out)
        for i in (1, 2):
            generated = client.load("file%d.gen" % i)
            self.assertEqual(generated, "CustomContent%d" % i)

    def test_export_template_generator(self):
        templated_generator = """
import os
from conans import ConanFile, load
from conans.model import Generator
class MyCustomTemplateGenerator(Generator):
    @property
    def filename(self):
        return "customfile.gen"
    @property
    def content(self):
        template = load(os.path.join(os.path.dirname(__file__), "mytemplate.txt"))
        return template % "Hello"

class MyCustomGeneratorWithTemplatePackage(ConanFile):
    exports = "mytemplate.txt"
"""
        client = TestClient()
        client.save({CONANFILE: templated_generator, "mytemplate.txt": "Template: %s"})
        client.run("create . gen/0.1@user/stable")

        client.run("install gen/0.1@user/stable -g=MyCustomTemplateGenerator")
        generated = client.load("customfile.gen")
        self.assertEqual(generated, "Template: Hello")

    def test_install_folder(self):
        # https://github.com/conan-io/conan/issues/5568
        templated_generator = textwrap.dedent("""
            from conans import ConanFile
            from conans.model import Generator
            class MyGenerator(Generator):
                @property
                def filename(self):
                    return "customfile.gen"
                @property
                def content(self):
                    return self.conanfile.install_folder

            class MyCustomGenerator(ConanFile):
                pass
            """)
        client = TestClient()
        client.save({CONANFILE: templated_generator, "mytemplate.txt": "Template: %s"})
        client.run("create . gen/0.1@user/stable")
        client.run("install gen/0.1@user/stable -g=MyGenerator")
        generated = client.load("customfile.gen")
        self.assertEqual(generated, client.current_folder)



import unittest
import tempfile
import textwrap
import os

import six

import conan.tools.qbs.qbsprofile as qbs

from conans import tools
from conans.errors import ConanException
from conans.test.utils.mocks import MockConanfile, MockSettings, MockOptions


class RunnerMock(object):
    class Expectation(object):
        def __init__(self, return_ok=True, output=None):
            self.return_ok = return_ok
            if six.PY2 and output:
                output = output.decode("utf-8")
            self.output = output

    def __init__(self, expectations=None):
        self.command_called = []
        self.expectations = expectations or [RunnerMock.Expectation()]

    def __call__(self, command, output, win_bash=False, subsystem=None):
        self.command_called.append(command)
        self.win_bash = win_bash
        self.subsystem = subsystem
        if not self.expectations:
            return 1
        expectation = self.expectations.pop(0)
        if expectation.output and output and hasattr(output, 'write'):
            output.write(expectation.output)
        return 0 if expectation.return_ok else 1


class MockConanfileWithFolders(MockConanfile):
    install_folder = tempfile.mkdtemp()

    def __del__(self):
        tools.rmdir(self.install_folder)

    def run(self, *args, **kwargs):
        if self.runner:
            if 'output' not in kwargs:
                kwargs['output'] = None
            self.runner(*args, **kwargs)


class QbsGenericTest(unittest.TestCase):
    def test_convert_bool(self):
        self.assertEqual(qbs._bool(True), 'true')
        self.assertEqual(qbs._bool(False), 'false')

    def test_convert_build_variant(self):
        conanfile = MockConanfileWithFolders(MockSettings({
            'os': 'Linux',
            'compiler': 'gcc'}))

        qbs_toolchain = qbs.QbsProfile(conanfile)
        self.assertEqual(qbs_toolchain._build_variant, None)

        for build_type, build_variant in qbs._build_variant.items():
            conanfile = MockConanfileWithFolders(MockSettings({
                'os': 'Linux',
                'compiler': 'gcc',
                'build_type': build_type}))

            qbs_toolchain = qbs.QbsProfile(conanfile)
            self.assertEqual(qbs_toolchain._build_variant, build_variant)

    def test_convert_architecture(self):
        conanfile = MockConanfileWithFolders(MockSettings({
            'os': 'Linux',
            'compiler': 'gcc'}))

        qbs_toolchain = qbs.QbsProfile(conanfile)
        self.assertEqual(qbs_toolchain._architecture, None)

        for arch, architecture in qbs._architecture.items():
            conanfile = MockConanfileWithFolders(MockSettings({
                'os': 'Linux',
                'compiler': 'gcc',
                'arch': arch}))

            qbs_toolchain = qbs.QbsProfile(conanfile)
            self.assertEqual(qbs_toolchain._architecture, architecture)

    def test_convert_optimization(self):
        conanfile = MockConanfileWithFolders(MockSettings({
            'os': 'Linux',
            'compiler': 'gcc'}))

        qbs_toolchain = qbs.QbsProfile(conanfile)
        self.assertEqual(qbs_toolchain._optimization, None)

        for build_type, optimization in qbs._optimization.items():
            conanfile = MockConanfileWithFolders(MockSettings({
                'os': 'Linux',
                'compiler': 'gcc',
                'build_type': build_type}))

            qbs_toolchain = qbs.QbsProfile(conanfile)
            self.assertEqual(qbs_toolchain._optimization, optimization)

    def test_use_sysroot_from_env(self):
        conanfile = MockConanfileWithFolders(MockSettings({
            'os': 'Linux',
            'compiler': 'gcc'}))

        sysroot = '/path/to/sysroot/foo/bar'
        with tools.environment_append({'SYSROOT': sysroot}):
            qbs_toolchain = qbs.QbsProfile(conanfile)
        self.assertEqual(qbs_toolchain._sysroot, sysroot)

    def test_detect_fpic_from_options(self):
        f_pic = {
            True: 'true',
            False: 'false',
            None: None
        }

        for option, value in f_pic.items():
            conanfile = MockConanfileWithFolders(MockSettings({
                    'os': 'Linux',
                    'compiler': 'gcc'
                }),
                MockOptions({
                    'fPIC': option
                }))

            qbs_toolchain = qbs.QbsProfile(conanfile)
            self.assertEqual(qbs_toolchain._position_independent_code, value)

    def test_convert_cxx_language_version(self):
        conanfile = MockConanfileWithFolders(MockSettings({
            'os': 'Linux',
            'compiler': 'gcc'}))

        qbs_toolchain = qbs.QbsProfile(conanfile)
        self.assertEqual(qbs_toolchain._cxx_language_version, None)
        conanfile = MockConanfileWithFolders(MockSettings({
            'os': 'Linux',
            'compiler': 'gcc',
            'compiler.cppstd': 17}))

        qbs_toolchain = qbs.QbsProfile(conanfile)
        self.assertEqual(qbs_toolchain._cxx_language_version, 'c++17')

        for cppstd, cxx_language_version in qbs._cxx_language_version.items():
            conanfile = MockConanfileWithFolders(MockSettings({
                'os': 'Linux',
                'compiler': 'gcc',
                'compiler.cppstd': cppstd}))

            qbs_toolchain = qbs.QbsProfile(conanfile)
            self.assertEqual(qbs_toolchain._cxx_language_version,
                             cxx_language_version)

    def test_convert_target_platform(self):
        conanfile = MockConanfileWithFolders(MockSettings({
            'compiler': 'gcc'}))

        qbs_toolchain = qbs.QbsProfile(conanfile)
        self.assertEqual(qbs_toolchain._target_platform, None)

        for os, target_platform in qbs._target_platform.items():
            conanfile = MockConanfileWithFolders(MockSettings({
                'os': os,
                'compiler': 'gcc'}))

            qbs_toolchain = qbs.QbsProfile(conanfile)
            self.assertEqual(qbs_toolchain._target_platform,
                             target_platform)

    def test_convert_runtime_library(self):
        conanfile = MockConanfileWithFolders(MockSettings({
            'compiler': 'Visual Studio'}))

        qbs_toolchain = qbs.QbsProfile(conanfile)
        self.assertEqual(qbs_toolchain._runtime_library, None)

        for runtime, runtime_library in qbs._runtime_library.items():
            conanfile = MockConanfileWithFolders(MockSettings({
                'compiler': 'Visual Studio',
                'compiler.runtime': runtime}))

            qbs_toolchain = qbs.QbsProfile(conanfile)
            self.assertEqual(qbs_toolchain._runtime_library,
                             runtime_library)

    def test_split_env_var_into_list(self):
        env_var_list = ['-p1', '-p2', '-p3_with_value=13',
                        '-p_with_space1="hello world"',
                        '"-p_with_space2=Hello World"']
        expected_list = ['-p1', '-p2', '-p3_with_value=13',
                         '-p_with_space1=hello world',
                         '-p_with_space2=Hello World']
        env_var = ' '.join(env_var_list)
        self.assertEqual(qbs._env_var_to_list(env_var), expected_list)

    def test_compiler_not_in_settings(self):
        conanfile = MockConanfile(MockSettings({}))
        with self.assertRaises(ConanException):
            qbs._check_for_compiler(conanfile)

    def test_compiler_in_settings_not_supported(self):
        conanfile = MockConanfile(
            MockSettings({'compiler': 'not realy a compiler name'}))
        with self.assertRaises(ConanException):
            qbs._check_for_compiler(conanfile)

    def test_valid_compiler(self):
        supported_compilers = ['Visual Studio', 'gcc', 'clang']
        for compiler in supported_compilers:
            conanfile = MockConanfile(MockSettings({'compiler': compiler}))
            qbs._check_for_compiler(conanfile)

    @staticmethod
    def _settings_to_test_against():
        return [
            {'os': 'Windows', 'compiler': 'gcc', 'compiler.version': '6', 'qbs_compiler': 'mingw'},
            {'os': 'Windows', 'compiler': 'clang', 'compiler.version': '3.9',
             'qbs_compiler': 'clang-cl'},
            {'os': 'Windows', 'compiler': 'Visual Studio', 'compiler.version': '15',
             'qbs_compiler': 'cl'},
            {'os': 'Windows', 'compiler': 'Visual Studio', 'compiler.version': '15',
             'compiler.toolset': 'ClangCL', 'qbs_compiler': 'clang-cl'},
            {'os': 'Windows', 'compiler': 'msvc', 'compiler.version': '19.0',
             'qbs_compiler': 'cl'},
            {'os': 'Linux', 'compiler': 'gcc', 'compiler.version': '6', 'qbs_compiler': 'gcc'},
            {'os': 'Linux', 'compiler': 'clang', 'compiler.version': '3.9', 'qbs_compiler': 'clang'}
        ]

    def test_convert_compiler_name_to_qbs_compiler_name(self):
        for settings in self._settings_to_test_against():
            def expected():
                return settings['qbs_compiler']
            conanfile = MockConanfile(MockSettings(settings))
            self.assertEqual(qbs._default_compiler_name(conanfile), expected())

    def test_settings_dir_location(self):
        conanfile = MockConanfileWithFolders(MockSettings({}))
        self.assertEqual(
            qbs._settings_dir(conanfile),
            '%s/conan_qbs_toolchain_settings_dir' % conanfile.install_folder)

    @unittest.skipIf(os.name == 'nt', "Test can only be performed with known MSVC version")
    def test_setup_toolchain_without_any_env_values(self):
        for settings in self._settings_to_test_against():
            conanfile = MockConanfileWithFolders(MockSettings(settings), runner=RunnerMock())
            qbs._setup_toolchains(conanfile)
            self.assertEqual(len(conanfile.runner.command_called), 1)
            self.assertEqual(
                conanfile.runner.command_called[0],
                'qbs-setup-toolchains --settings-dir "%s" %s %s' % (
                    qbs._settings_dir(conanfile), settings['qbs_compiler'],
                    qbs._profile_name))

    def test_setup_toolchain_with_compiler_from_env(self):
        compiler = 'compiler_from_env'
        for settings in self._settings_to_test_against():
            conanfile = MockConanfileWithFolders(MockSettings(settings), runner=RunnerMock())
            with tools.environment_append({'CC': compiler}):
                qbs._setup_toolchains(conanfile)
            self.assertEqual(len(conanfile.runner.command_called), 1)
            self.assertEqual(
                conanfile.runner.command_called[0],
                'qbs-setup-toolchains --settings-dir "%s" %s %s' % (
                    qbs._settings_dir(conanfile), compiler,
                    qbs._profile_name))

    def test_linker_flags_parser(self):
        test_data_ld_flags = {
            '-Wl,flag1': ([], ['flag1']),
            '-Wl,flag1,flag2': ([], ['flag1', 'flag2']),
            '-Wl,flag1 -Wl,flag2': ([], ['flag1', 'flag2']),
            '-dFlag1': (['-dFlag1'], []),
            '-dFlag1 -dFlag2': (['-dFlag1', '-dFlag2'], []),
            '-Wl,flag1 -dFlag1': (['-dFlag1'], ['flag1']),
            '-Wl,flag1,flag2 -dFlag1': (['-dFlag1'], ['flag1', 'flag2']),
            '-Wl,flag1,flag2 -dFlag1 -Wl,flag3 -dFlag2 -dFlag3 -Wl,flag4,flag5':
                (['-dFlag1', '-dFlag2', '-dFlag3'],
                 ['flag1', 'flag2', 'flag3', 'flag4', 'flag5']),
        }
        for ld_flags, expected in test_data_ld_flags.items():
            driver_linker_flags, linker_flags = expected
            parser = qbs.LinkerFlagsParser(qbs._env_var_to_list(ld_flags))
            self.assertEqual(parser.driver_linker_flags,
                             driver_linker_flags)
            self.assertEqual(parser.linker_flags,
                             linker_flags)

    @staticmethod
    def _generate_flags(flag, qbs_key):
        return {'env': ('-{0}1 -{0}2 -{0}3_with_value=13 '
                        '-{0}_with_space="hello world"').format(flag),
                'qbs_value': ("['-{0}1', '-{0}2', '-{0}3_with_value=13', "
                              "'-{0}_with_space=hello world']").format(flag),
                'qbs_key': qbs_key}

    def test_flags_from_env(self):
        asm = self._generate_flags('asm', 'assemblerFlags')
        c = self._generate_flags('c', 'cFlags')
        cpp = self._generate_flags('cpp', 'cppFlags')
        cxx = self._generate_flags('cxx', 'cxxFlags')
        wl = self._generate_flags('Wl,', 'linkerFlags')
        ld = self._generate_flags('ld', 'driverLinkerFlags')
        env = {
            'ASFLAGS': asm['env'],
            'CFLAGS': c['env'],
            'CPPFLAGS': cpp['env'],
            'CXXFLAGS': cxx['env'],
            'LDFLAGS': '%s %s' % (wl['env'], ld['env'])
        }
        with tools.environment_append(env):
            flags_from_env = qbs._flags_from_env()

        expected_flags = {
            'cpp.'+asm['qbs_key']: asm['qbs_value'],
            'cpp.'+c['qbs_key']: c['qbs_value'],
            'cpp.'+cpp['qbs_key']: cpp['qbs_value'],
            'cpp.'+cxx['qbs_key']: cxx['qbs_value'],
            'cpp.'+wl['qbs_key']: wl['qbs_value'].replace('-Wl,', ''),
            'cpp.'+ld['qbs_key']: ld['qbs_value']
        }
        self.assertEqual(flags_from_env, expected_flags)

    @staticmethod
    def _generate_qbs_config_output():
        return textwrap.dedent('''\
            profiles.conan.cpp.cCompilerName: "gcc"
            profiles.conan.cpp.compilerName: "g++"
            profiles.conan.cpp.cxxCompilerName: "g++"
            profiles.conan.cpp.driverFlags: \
            ["-march=armv7e-m", "-mtune=cortex-m4", "--specs=nosys.specs"]
            profiles.conan.cpp.platformCommonCompilerFlags: undefined
            profiles.conan.cpp.platformLinkerFlags: undefined
            profiles.conan.cpp.toolchainInstallPath: "/usr/bin"
            profiles.conan.cpp.toolchainPrefix: "arm-none-eabi-"
            profiles.conan.qbs.someBoolProp: "true"
            profiles.conan.qbs.someIntProp: "13"
            profiles.conan.qbs.toolchain: ["gcc"]
            ''')

    def test_read_qbs_toolchain_from_qbs_config_output(self):
        expected_config = {
            'cpp.cCompilerName': '"gcc"',
            'cpp.compilerName': '"g++"',
            'cpp.cxxCompilerName': '"g++"',
            'cpp.driverFlags': '["-march=armv7e-m", "-mtune=cortex-m4", "--specs=nosys.specs"]',
            'cpp.platformCommonCompilerFlags': 'undefined',
            'cpp.platformLinkerFlags': 'undefined',
            'cpp.toolchainInstallPath': '"/usr/bin"',
            'cpp.toolchainPrefix': '"arm-none-eabi-"',
            'qbs.someBoolProp': 'true',
            'qbs.someIntProp': '13',
            'qbs.toolchain': '["gcc"]'
        }

        conanfile = MockConanfileWithFolders(
            MockSettings({}), runner=RunnerMock(
                expectations=[RunnerMock.Expectation(
                    output=self._generate_qbs_config_output())]))
        config = qbs._read_qbs_toolchain_from_config(conanfile)
        self.assertEqual(len(conanfile.runner.command_called), 1)
        self.assertEqual(conanfile.runner.command_called[0],
                         'qbs-config --settings-dir "%s" --list' % (
                            qbs._settings_dir(conanfile)))
        self.assertEqual(config, expected_config)

    @unittest.skipIf(six.PY2, "Order of qbs output is defined only for PY3")
    def test_toolchain_content(self):
        expected_content = textwrap.dedent('''\
            import qbs

            Project {
                Profile {
                    name: "conan_toolchain_profile"

                    /* detected via qbs-setup-toolchains */
                    cpp.cCompilerName: "gcc"
                    cpp.compilerName: "g++"
                    cpp.cxxCompilerName: "g++"
                    cpp.driverFlags: ["-march=armv7e-m", "-mtune=cortex-m4", "--specs=nosys.specs"]
                    cpp.platformCommonCompilerFlags: undefined
                    cpp.platformLinkerFlags: undefined
                    cpp.toolchainInstallPath: "/usr/bin"
                    cpp.toolchainPrefix: "arm-none-eabi-"
                    qbs.someBoolProp: true
                    qbs.someIntProp: 13
                    qbs.toolchain: ["gcc"]

                    /* deduced from environment */
                    qbs.sysroot: "/foo/bar/path"

                    /* conan settings */
                    qbs.buildVariant: "release"
                    qbs.architecture: "x86_64"
                    qbs.targetPlatform: "linux"
                    qbs.optimization: "small"
                    cpp.cxxLanguageVersion: "c++17"

                    /* package options */
                    cpp.positionIndependentCode: true
                }
            }''')

        conanfile = MockConanfileWithFolders(
            MockSettings({
                'compiler': 'gcc',
                'compiler.cppstd': 17,
                'os': 'Linux',
                'build_type': 'MinSizeRel',
                'arch': 'x86_64'
            }),
            options=MockOptions({
                'fPIC': True
            }),
            runner=RunnerMock(
                expectations=[
                    RunnerMock.Expectation(),
                    RunnerMock.Expectation(
                        output=self._generate_qbs_config_output()),
                ]))

        with tools.environment_append({'SYSROOT': '/foo/bar/path'}):
            qbs_toolchain = qbs.QbsProfile(conanfile)

        self.assertEqual(qbs_toolchain.content, expected_content)

    @staticmethod
    def _generate_qbs_config_output_msvc():
        return textwrap.dedent('''\
            profiles.conan.cpp.compilerVersion: "19.28.29333"
            profiles.conan.cpp.toolchainInstallPath: \
            "C:/Program Files (x86)/Microsoft Visual Studio/2019/Community/VC/Tools/MSVC/14.28.29333/bin/Hostx64/x64"
            profiles.conan.qbs.architecture: "x86_64"
            profiles.conan.qbs.targetPlatform: "windows"
            profiles.conan.qbs.toolchainType: "msvc"
            profiles.conan.cpp.driverFlags: \
            ["-march=armv7e-m", "-mtune=cortex-m4", "--specs=nosys.specs"]
            profiles.conan.qbs.someBoolProp: "true"
            profiles.conan.qbs.someIntProp: "13"
            ''')

    def test_read_qbs_toolchain_from_qbs_config_output_msvc(self):
        expected_config = {
            'cpp.compilerVersion': '"19.28.29333"',
            'cpp.toolchainInstallPath': '"C:/Program Files (x86)/Microsoft Visual Studio/2019/Community/VC/Tools/MSVC/14.28.29333/bin/Hostx64/x64"',
            'qbs.architecture': '"x86_64"',
            'qbs.targetPlatform': '"windows"',
            'qbs.toolchainType': '"msvc"',
            'cpp.driverFlags': '["-march=armv7e-m", "-mtune=cortex-m4", "--specs=nosys.specs"]',
            'qbs.someBoolProp': 'true',
            'qbs.someIntProp': '13',
        }

        conanfile = MockConanfileWithFolders(
            MockSettings({}), runner=RunnerMock(
                expectations=[RunnerMock.Expectation(
                    output=self._generate_qbs_config_output_msvc())]))
        config = qbs._read_qbs_toolchain_from_config(conanfile)
        self.assertEqual(len(conanfile.runner.command_called), 1)
        self.assertEqual(conanfile.runner.command_called[0],
                         'qbs-config --settings-dir "%s" --list' % (
                            qbs._settings_dir(conanfile)))
        self.assertEqual(config, expected_config)

    @unittest.skipIf(six.PY2, "Order of qbs output is defined only for PY3")
    def test_toolchain_content_msvc(self):
        expected_content = textwrap.dedent('''\
            import qbs

            Project {
                Profile {
                    name: "conan_toolchain_profile"

                    /* detected via qbs-setup-toolchains */
                    cpp.compilerVersion: "19.28.29333"
                    cpp.toolchainInstallPath: "C:/Program Files (x86)/Microsoft Visual Studio/2019/Community/VC/Tools/MSVC/14.28.29333/bin/Hostx64/x64"
                    qbs.architecture: "x86_64"
                    qbs.targetPlatform: "windows"
                    qbs.toolchainType: "msvc"
                    cpp.driverFlags: ["-march=armv7e-m", "-mtune=cortex-m4", "--specs=nosys.specs"]
                    qbs.someBoolProp: true
                    qbs.someIntProp: 13

                    /* deduced from environment */
                    qbs.sysroot: "/foo/bar/path"

                    /* conan settings */
                    qbs.buildVariant: "release"
                    qbs.optimization: "small"
                    cpp.runtimeLibrary: "dynamic"

                    /* package options */
                    cpp.positionIndependentCode: true
                }
            }''')

        conanfile = MockConanfileWithFolders(
            MockSettings({
                'compiler': 'Visual Studio',
                'compiler.version': 15,
                'compiler.runtime': 'MD',
                'os': 'Windows',
                'build_type': 'MinSizeRel',
                'arch': 'x86_64',
            }),
            options=MockOptions({
                'fPIC': True
            }),
            runner=RunnerMock(
                expectations=[
                    RunnerMock.Expectation(),
                    RunnerMock.Expectation(
                        output=self._generate_qbs_config_output_msvc()),
                ]))

        with tools.environment_append({'SYSROOT': '/foo/bar/path'}):
            qbs_toolchain = qbs.QbsProfile(conanfile)

        self.assertEqual(qbs_toolchain.content, expected_content)



import unittest
from parameterized import parameterized

from conans.client.conf.compiler_id import detect_compiler_id, CompilerId, UNKNOWN_COMPILER, \
    GCC, LLVM_GCC, CLANG, APPLE_CLANG, SUNCC, VISUAL_STUDIO, MSVC, INTEL, QCC, MCST_LCC
from conans.test.unittests.util.tools_test import RunnerMock


class CompilerIdTest(unittest.TestCase):
    def test_no_output(self):
        compiler_id = detect_compiler_id("gcc", runner=RunnerMock())
        self.assertEqual(UNKNOWN_COMPILER, compiler_id)

    def test_bad_output(self):
        runner = RunnerMock()
        runner.output = "#define UNKNOWN 1"
        compiler_id = detect_compiler_id("gcc", runner=runner)
        self.assertEqual(UNKNOWN_COMPILER, compiler_id)

    def test_bad_numbers(self):
        runner = RunnerMock()
        runner.output = "#define __GNUC__ A\n" \
                        "#define __GNUC_MINOR__ B\n" \
                        "#define __GNUC_PATCHLEVEL__ C\n"
        compiler_id = detect_compiler_id("gcc", runner=runner)
        self.assertEqual(UNKNOWN_COMPILER, compiler_id)

    def test_incomplete(self):
        runner = RunnerMock()
        runner.output = "#define __GNUC__ 7"
        compiler_id = detect_compiler_id("gcc", runner=runner)
        self.assertEqual(UNKNOWN_COMPILER, compiler_id)

    def test_gcc(self):
        runner = RunnerMock()
        runner.output = "#define __GNUC__ 7\n" \
                        "#define __GNUC_MINOR__ 3\n" \
                        "#define __GNUC_PATCHLEVEL__ 0\n"
        compiler_id = detect_compiler_id("gcc", runner=runner)
        self.assertEqual(CompilerId(GCC, 7, 3, 0), compiler_id)

    def test_llvm_gcc(self):
        runner = RunnerMock()
        runner.output = "#define __llvm__ 1\n" \
                        "#define __GNUC__ 7\n" \
                        "#define __GNUC_MINOR__ 3\n" \
                        "#define __GNUC_PATCHLEVEL__ 0\n"
        compiler_id = detect_compiler_id("gcc", runner=runner)
        self.assertEqual(CompilerId(LLVM_GCC, 7, 3, 0), compiler_id)

    def test_clang(self):
        runner = RunnerMock()
        # clang defines __GNUC__ and may define _MSC_VER as well (on Windows)
        runner.output = "#define _MSC_VER 1922\n" \
                        "#define __GNUC__ 4\n" \
                        "#define __GNUC_MINOR__ 1\n" \
                        "#define __GNUC_PATCHLEVEL__ 1\n" \
                        "#define __clang__ 1\n"\
                        "#define __clang_major__ 9\n" \
                        "#define __clang_minor__ 0\n" \
                        "#define __clang_patchlevel__ 1\n"
        compiler_id = detect_compiler_id("clang", runner=runner)
        self.assertEqual(CompilerId(CLANG, 9, 0, 1), compiler_id)

    def test_apple_clang(self):
        runner = RunnerMock()
        runner.output = "#define __GNUC__ 4\n" \
                        "#define __GNUC_MINOR__ 1\n" \
                        "#define __GNUC_PATCHLEVEL__ 1\n" \
                        "#define __clang__ 1\n"\
                        "#define __clang_major__ 10\n" \
                        "#define __clang_minor__ 0\n" \
                        "#define __clang_patchlevel__ 1\n" \
                        "#define __apple_build_version__ 10010046\n"
        compiler_id = detect_compiler_id("clang", runner=runner)
        self.assertEqual(CompilerId(APPLE_CLANG, 10, 0, 1), compiler_id)

    def test_suncc(self):
        runner = RunnerMock()
        runner.output = "#define __SUNPRO_CC 0x450\n"
        compiler_id = detect_compiler_id("suncc", runner=runner)
        self.assertEqual(CompilerId(SUNCC, 4, 5, 0), compiler_id)

    @parameterized.expand([("MSC_CMD_FLAGS=-D_MSC_VER=1400", 8, 0, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1500", 9, 0, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1600", 10, 0, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1700", 11, 0, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1800", 12, 0, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1900", 14, 0, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1910", 15, 0, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1911", 15, 3, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1912", 15, 5, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1913", 15, 6, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1914", 15, 7, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1915", 15, 8, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1916", 15, 9, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1921", 16, 1, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1922", 16, 2, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1923", 16, 3, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1924", 16, 4, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1925", 16, 5, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1926", 16, 6, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1927", 16, 7, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1928", 16, 8, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1928 -D_MSC_FULL_VER=192829500", 16, 9, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1929", 16, 10, 0),
                           ("MSC_CMD_FLAGS=-D_MSC_VER=1929 -D_MSC_FULL_VER=192930100", 16, 11, 0),
                           ])
    def test_visual_studio(self, line, major, minor, patch):
        runner = RunnerMock()
        runner.output = line
        compiler_id = detect_compiler_id("cl", runner=runner)
        self.assertEqual(CompilerId(VISUAL_STUDIO, major, minor, patch), compiler_id)

    def test_msvc(self):
        runner = RunnerMock()
        runner.output = "MSC_CMD_FLAGS=-D_MSC_VER=1930"
        compiler_id = detect_compiler_id("cl", runner=runner)
        self.assertEqual(CompilerId(MSVC, 19, 30, 0), compiler_id)

    def test_intel(self):
        runner = RunnerMock()
        # Intel C++ may define __GNUC__ and _MSC_VER for compatibility
        runner.output = "#define _MSC_VER 1922\n" \
                        "#define __GNUC__ 4\n" \
                        "#define __GNUC_MINOR__ 1\n" \
                        "#define __GNUC_PATCHLEVEL__ 1\n" \
                        "#define __INTEL_COMPILER 1900\n" \
                        "#define __INTEL_COMPILER_UPDATE 3\n"
        compiler_id = detect_compiler_id("clang", runner=runner)
        self.assertEqual(CompilerId(INTEL, 19, 0, 3), compiler_id)

    def test_qcc(self):
        runner = RunnerMock()
        runner.output = "#define __QNX__ 1\n" \
                        "#define __GNUC__ 4\n" \
                        "#define __GNUC_MINOR__ 2\n" \
                        "#define __GNUC_PATCHLEVEL__ 4\n"
        compiler_id = detect_compiler_id("qcc", runner=runner)
        self.assertEqual(CompilerId(QCC, 4, 2, 4), compiler_id)

    def test_mcst_lcc(self):
        runner = RunnerMock()
        runner.output = "#define __LCC__ 125\n" \
                        "#define __LCC_MINOR__ 6\n" \
                        "#define __e2k__ 1\n"
        compiler_id = detect_compiler_id("lcc", runner=runner)
        self.assertEqual(CompilerId(MCST_LCC, 1, 25, 6), compiler_id)



import os
import platform
import textwrap

from conans.model.ref import ConanFileReference
from conans.test.assets.sources import gen_function_cpp, gen_function_h
from conans.test.functional.toolchains.meson._base import TestMesonBase


class MesonToolchainTest(TestMesonBase):
    _conanfile_py = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.meson import Meson, MesonToolchain


    class App(ConanFile):
        settings = "os", "arch", "compiler", "build_type"
        options = {"shared": [True, False], "fPIC": [True, False]}
        default_options = {"shared": False, "fPIC": True}

        def config_options(self):
            if self.settings.os == "Windows":
                del self.options.fPIC

        def layout(self):
            self.folders.generators = 'build/gen_folder'
            self.folders.build = "build"

        def generate(self):
            tc = MesonToolchain(self)
            tc.project_options["STRING_DEFINITION"] = "Text"
            tc.project_options["TRUE_DEFINITION"] = True
            tc.project_options["FALSE_DEFINITION"] = False
            tc.project_options["INT_DEFINITION"] = 42
            tc.project_options["ARRAY_DEFINITION"] = ["Text1", "Text2"]
            tc.generate()

        def build(self):
            meson = Meson(self)
            meson.configure()
            meson.build(target='hello')
            meson.build(target='demo')
    """)

    _meson_options_txt = textwrap.dedent("""
    option('STRING_DEFINITION', type : 'string', description : 'a string option')
    option('INT_DEFINITION', type : 'integer', description : 'an integer option', value: 0)
    option('FALSE_DEFINITION', type : 'boolean', description : 'a boolean option (false)')
    option('TRUE_DEFINITION', type : 'boolean', description : 'a boolean option (true)')
    option('ARRAY_DEFINITION', type : 'array', description : 'an array option')
    option('HELLO_MSG', type : 'string', description : 'message to print')
    """)

    _meson_build = textwrap.dedent("""
    project('tutorial', 'cpp')
    add_global_arguments('-DSTRING_DEFINITION="' + get_option('STRING_DEFINITION') + '"',
                         language : 'cpp')
    add_global_arguments('-DHELLO_MSG="' + get_option('HELLO_MSG') + '"', language : 'cpp')
    hello = library('hello', 'hello.cpp')
    executable('demo', 'main.cpp', link_with: hello)
    """)

    def test_definition_of_global_options(self):
        hello_h = gen_function_h(name="hello")
        hello_cpp = gen_function_cpp(name="hello", preprocessor=["STRING_DEFINITION"])
        app = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])

        self.t.save({"conanfile.py": self._conanfile_py,
                     "meson.build": self._meson_build,
                     "meson_options.txt": self._meson_options_txt,
                     "hello.h": hello_h,
                     "hello.cpp": hello_cpp,
                     "main.cpp": app})

        self.t.run("install . %s" % self._settings_str)

        content = self.t.load(os.path.join("build", "gen_folder", "conan_meson_native.ini"))

        self.assertIn("[project options]", content)
        self.assertIn("STRING_DEFINITION = 'Text'", content)
        self.assertIn("TRUE_DEFINITION = true", content)
        self.assertIn("FALSE_DEFINITION = false", content)
        self.assertIn("INT_DEFINITION = 42", content)
        self.assertIn("ARRAY_DEFINITION = ['Text1', 'Text2']", content)

        self.assertIn("[built-in options]", content)
        self.assertIn("buildtype = 'release'", content)

        self.t.run("build .")
        self.t.run_command(os.path.join("build", "demo"))

        self.assertIn("hello: Release!", self.t.out)
        self.assertIn("STRING_DEFINITION: Text", self.t.out)

        self.assertIn("[properties]", content)
        self.assertNotIn("needs_exe_wrapper", content)

        self._check_binary()

    def test_meson_default_dirs(self):
        self.t.run("new hello/1.0 -m meson_exe")
        # self.t.run("new meson_exe -d name=hello -d version=1.0 -m meson_exe")

        meson_build = textwrap.dedent("""
        project('tutorial', 'cpp')
        # Creating specific library
        hello = library('hello', 'src/hello.cpp', install: true)
        # Creating specific executable
        executable('demo', 'src/main.cpp', link_with: hello, install: true)
        # Creating specific data in src/ (they're going to be exported)
        install_data(['src/file1.txt', 'src/file2.txt'])
        """)
        conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.meson import Meson
        class HelloConan(ConanFile):
            name = "hello"
            version = "1.0"
            settings = "os", "compiler", "build_type", "arch"
            exports_sources = "meson.build", "src/*"
            generators = "MesonToolchain"

            def layout(self):
                self.folders.build = "build"
                # Only adding "res" to resdirs
                self.cpp.package.resdirs = ["res"]

            def build(self):
                meson = Meson(self)
                meson.configure()
                meson.build()

            def package(self):
                meson = Meson(self)
                meson.install()
        """)
        test_conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.build import cross_building
        class HelloTestConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "VirtualRunEnv"
            apply_env = False

            def test(self):
                if not cross_building(self):
                    self.run("demo", env="conanrun")
        """)
        # Replace meson.build, conanfile.py and test_package/conanfile.py
        self.t.save({"meson.build": meson_build,
                     "conanfile.py": conanfile,
                     "test_package/conanfile.py": test_conanfile,
                     "src/file1.txt": "", "src/file2.txt": ""})
        self.t.run("create .")
        # Check if all the files are in the final directories
        ref = ConanFileReference.loads("hello/1.0")
        cache_package_folder = self.t.cache.package_layout(ref).packages()
        package_folder = os.path.join(cache_package_folder, os.listdir(cache_package_folder)[0])
        if platform.system() == "Windows":
            assert os.path.exists(os.path.join(package_folder, "lib", "hello.lib"))
            assert os.path.exists(os.path.join(package_folder, "bin", "hello.dll"))
            assert os.path.exists(os.path.join(package_folder, "bin", "demo.exe"))
        else:
            ext = "dylib" if platform.system() == "Darwin" else "so"
            assert os.path.exists(os.path.join(package_folder, "bin", "demo"))
            assert os.path.exists(os.path.join(package_folder, "lib", "libhello." + ext))
        # res/tutorial -> tutorial is being added automatically by Meson
        assert os.path.exists(os.path.join(package_folder, "res", "tutorial", "file1.txt"))
        assert os.path.exists(os.path.join(package_folder, "res", "tutorial", "file2.txt"))



import textwrap

from conans.test.utils.conan_v2_tests import ConanV2ModeTestCase


class CppInfoTestCase(ConanV2ModeTestCase):

    def test_cpp_info_name(self):
        t = self.get_client()
        lib = textwrap.dedent("""
            from conans import ConanFile

            class Recipe(ConanFile):
                name = "lib"

                def package_info(self):
                    self.cpp_info.name = "lib_name"
        """)
        app = textwrap.dedent("""
            from conans import ConanFile

            class Recipe(ConanFile):
                name = "app"
                requires = "lib/version"

                def build(self):
                    lib_cpp_info = self.deps_cpp_info["lib"]
                    self.output.info("get_name: {}".format(lib_cpp_info.get_name('cmake')))
                    self.output.info("name: {}".format(lib_cpp_info.name))
        """)

        t.save({'lib.py': lib,
                'app.py': app})
        t.run('export lib.py lib/version@')
        t.run('create app.py app/version@ --build=missing', assert_error=True)

        self.assertIn("app/version: get_name: lib_name", t.out)
        self.assertIn("ConanV2Exception: Conan v2 incompatible: Use 'get_name(generator)' instead",
                      t.out)



import os
import re

from jinja2 import Template

from conans import __version__ as client_version
from conans.client.cmd.new_ci import ci_get_files
from conans.errors import ConanException
from conans.model.ref import ConanFileReference, get_reference_fields
from conans.util.files import load
from conans.client.cmd.new import (conanfile_bare, 
                                   conanfile_sources, conanfile_header,
                                   test_conanfile, test_cmake,
                                   test_cmake_pure_c, test_main,
                                   hello_c, hello_h,
                                   hello_cpp, cmake_pure_c,
                                   cmake, gitignore_template,
                                   _render_template, 
                                   _get_files_from_template_dir)

conanfile = """from conans import CMake, tools
from sense2 import Sense2File
import os

class {package_name}Conan(Sense2File):
    name = "{name}"
    version = "{version}"
    # license = "<Put the package license here>"
    # author = "<Put your name here> <And your email here>"
    # url = "<Package recipe repository url here, for issues about the package>"
    description = "<Description of {package_name} here>"
    # topics = ("<Put some tag here>", "<here>", "<and here>")
    settings = "os", "compiler", "build_type", "arch"
    options = {{"shared": [True, False], "fPIC": [True, False]}}
    default_options = {{"shared": False, "fPIC": True}}
    generators = "cmake"
    # requires = ""
    # docker_version = "<dokcer version>"
    # sense_system_deb = "<debs>"
    
    ## make deb setting
    deb_control = {{
       "Package": name,
       "Version": version,
       "Maintainer": "tester",
       "Architecture": "amd64",
       "Description": "descriptionn",
       "Section": "custom",
       "Priority": "optional",
       "Essential": "no",
       "Depends": "depends",
       "Homepage": "www.example.com",
    }}
    changelog = "my changelog test"
    
    ### upload cpp package to artifactory
    # user="user"
    # channel="beta"

    ## upload deb setting
    apt_user = "qiuhai"
    apt_sever = "http://10.0.50.109:8081/repository/sense2-system-deb/"
    
    ## set source at the same folder with recipe
    exports_sources = "*"

#     def source(self):
#         # self.run("git clone http://114.115.150.64:8080/open_source/hello.git")
#         # self.run("cd hello")
#         # This small hack might be useful to guarantee proper /MT /MD linkage
#         # in MSVC if the packaged project doesn't have variables to set it
#         # properly
# #         tools.replace_in_file("CMakeLists.txt", "project(MD5Encrypter)",
# #                               '''project(MD5Encrypter)
# # include(${{CMAKE_BINARY_DIR}}/conanbuildinfo.cmake)
# # conan_basic_setup()''')
#         pass

    def build(self):
        cmake = CMake(self)
        cmake.configure(source_folder=".")
        cmake.build()
        pass

    def package(self):
        self.copy("*.h", dst="include", src="hello")
        self.copy("*.so", dst="lib", keep_path=False)
        self.copy("*.a", dst="lib", keep_path=False)
        self.copy("*", dst="bin", src="bin")

    # def package_info(self):
    #    self.cpp_info.libs = ["hello"]
        
    def deb(self):
        ## copy *.deb to deb folder
        package_name = self.name
        deb_file = package_name + ".deb"
        self.copy(deb_file, dst=".", keep_path=False)
        pass
        
    def install_deb(self):
        print("run conanfile install_deb")
        pass
    
    def install_docker(self):
        print("run conanfile install_docker")
        pass

    def install_sense_system(self):
        print("run conanfile install_sense_system")
        pass
"""


def cmd_new(ref, header=False, pure_c=False, test=False, exports_sources=False, bare=False,
            visual_versions=None, linux_gcc_versions=None, linux_clang_versions=None,
            osx_clang_versions=None, shared=None, upload_url=None, gitignore=None,
            gitlab_gcc_versions=None, gitlab_clang_versions=None,
            circleci_gcc_versions=None, circleci_clang_versions=None, circleci_osx_versions=None,
            template=None, cache=None, defines=None):
    try:
        name, version, user, channel, revision = get_reference_fields(ref, user_channel_input=False)
        # convert "package_name" -> "PackageName"
        package_name = re.sub(r"(?:^|[\W_])(\w)", lambda x: x.group(1).upper(), name)
    except ValueError:
        raise ConanException("Bad parameter, please use full package name,"
                             "e.g.: MyLib/1.2.3@user/testing")

    # Validate it is a valid reference
    ConanFileReference(name, version, user, channel)

    if header and exports_sources:
        raise ConanException("'header' and 'sources' are incompatible options")
    if pure_c and header:
        raise ConanException("'pure_c' is incompatible with 'header'")
    if pure_c and not exports_sources:
        raise ConanException("'pure_c' requires the use of --source")
    if bare and (header or exports_sources):
        raise ConanException("'bare' is incompatible with 'header' and 'sources'")
    if template and (header or exports_sources or bare or pure_c):
        raise ConanException("'template' is incompatible with 'header', "
                             "'sources', 'pure-c' and 'bare'")

    defines = defines or dict()

    if header:
        files = {"conanfile.py": conanfile_header.format(name=name, version=version,
                                                         package_name=package_name)}
    elif exports_sources:
        if not pure_c:
            files = {"conanfile.py": conanfile_sources.format(name=name, version=version,
                                                              package_name=package_name,
                                                              configure=""),
                     "src/{}.cpp".format(name): hello_cpp.format(name=name, version=version),
                     "src/{}.h".format(name): hello_h.format(name=name, version=version),
                     "src/CMakeLists.txt": cmake.format(name=name, version=version)}
        else:
            config = ("\n    def configure(self):\n"
                      "        del self.settings.compiler.libcxx\n"
                      "        del self.settings.compiler.cppstd\n")
            files = {"conanfile.py": conanfile_sources.format(name=name, version=version,
                                                              package_name=package_name,
                                                              configure=config),
                     "src/{}.c".format(name): hello_c.format(name=name, version=version),
                     "src/{}.h".format(name): hello_h.format(name=name, version=version),
                     "src/CMakeLists.txt": cmake_pure_c.format(name=name, version=version)}
    elif bare:
        files = {"conanfile.py": conanfile_bare.format(name=name, version=version,
                                                       package_name=package_name)}
    elif template:
        is_file_template = os.path.basename(template).endswith('.py')
        if is_file_template:
            if not os.path.isabs(template):
                # FIXME: Conan 2.0. The old path should be removed
                old_path = os.path.join(cache.cache_folder, "templates", template)
                new_path = os.path.join(cache.cache_folder, "templates", "command/new", template)
                template = new_path if os.path.isfile(new_path) else old_path
            if not os.path.isfile(template):
                raise ConanException("Template doesn't exist: %s" % template)
            replaced = _render_template(load(template),
                                        name=name,
                                        version=version,
                                        package_name=package_name,
                                        defines=defines)
            files = {"conanfile.py": replaced}
        elif template == "cmake_lib":
            from conans.assets.templates.new_v2_cmake import get_cmake_lib_files
            files = get_cmake_lib_files(name, version, package_name)
        elif template == "cmake_exe":
            from conans.assets.templates.new_v2_cmake import get_cmake_exe_files
            files = get_cmake_exe_files(name, version, package_name)
        elif template == "meson_lib":
            from conans.assets.templates.new_v2_meson import get_meson_lib_files
            files = get_meson_lib_files(name, version, package_name)
        elif template == "meson_exe":
            from conans.assets.templates.new_v2_meson import get_meson_exe_files
            files = get_meson_exe_files(name, version, package_name)
        elif template == "msbuild_lib":
            from conans.assets.templates.new_v2_msbuild import get_msbuild_lib_files
            files = get_msbuild_lib_files(name, version, package_name)
        elif template == "msbuild_exe":
            from conans.assets.templates.new_v2_msbuild import get_msbuild_exe_files
            files = get_msbuild_exe_files(name, version, package_name)
        elif template == "bazel_lib":
            from conans.assets.templates.new_v2_bazel import get_bazel_lib_files
            files = get_bazel_lib_files(name, version, package_name)
        elif template == "bazel_exe":
            from conans.assets.templates.new_v2_bazel import get_bazel_exe_files
            files = get_bazel_exe_files(name, version, package_name)
        elif template == "autotools_lib":
            from conans.assets.templates.new_v2_autotools import get_autotools_lib_files
            files = get_autotools_lib_files(name, version, package_name)
        elif template == "autotools_exe":
            from conans.assets.templates.new_v2_autotools import get_autotools_exe_files
            files = get_autotools_exe_files(name, version, package_name)
        else:
            if not os.path.isabs(template):
                template = os.path.join(cache.cache_folder, "templates", "command/new", template)
            if not os.path.isdir(template):
                raise ConanException("Template doesn't exist: {}".format(template))
            template = os.path.normpath(template)
            files = _get_files_from_template_dir(template_dir=template,
                                                 name=name,
                                                 version=version,
                                                 package_name=package_name,
                                                 defines=defines)
    else:
        files = {"conanfile.py": conanfile.format(name=name, version=version,
                                                  package_name=package_name)}

    if test:
        files["test_package/conanfile.py"] = test_conanfile.format(name=name, version=version,
                                                                   user=user, channel=channel,
                                                                   package_name=package_name)
        if pure_c:
            files["test_package/example.c"] = test_main.format(name=name)
            files["test_package/CMakeLists.txt"] = test_cmake_pure_c
        else:
            include_name = name if exports_sources else "hello"
            files["test_package/example.cpp"] = test_main.format(name=include_name)
            files["test_package/CMakeLists.txt"] = test_cmake

    if gitignore:
        files[".gitignore"] = gitignore_template

    files.update(ci_get_files(name, version, user, channel, visual_versions,
                              linux_gcc_versions, linux_clang_versions,
                              osx_clang_versions, shared, upload_url,
                              gitlab_gcc_versions, gitlab_clang_versions,
                              circleci_gcc_versions, circleci_clang_versions,
                              circleci_osx_versions))
    return files




import textwrap

from conan.tools.meson import Meson
from conans.model.conf import ConfDefinition
from conans.test.utils.mocks import ConanFileMock, MockSettings


def test_meson_build():
    c = ConfDefinition()
    c.loads(textwrap.dedent("""\
        tools.build:jobs=10
    """))

    settings = MockSettings({"build_type": "Release",
                             "compiler": "gcc",
                             "compiler.version": "7",
                             "os": "Linux",
                             "arch": "x86_64"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    conanfile.display_name = 'test'
    conanfile.conf = c.get_conanfile_conf(None)

    meson = Meson(conanfile)
    meson.build()

    assert '-j10' in str(conanfile.command)



# -*- coding: utf-8 -*-
import textwrap
import unittest

from jinja2 import Template

from conans.test.utils.tools import TestClient


class WorkspaceTest(unittest.TestCase):
    conanfile = Template(textwrap.dedent("""
        from conans import ConanFile

        class Library(ConanFile):
            name = "{{ name }}"
            version = "{{ version }}"
            {% if br %}build_requires = "{{ br }}"{% endif %}
            {% if req %}requires = "{{ req }}"{% endif %}
    """))

    def test_graph_failure(self):
        # https://github.com/conan-io/conan/issues/5612
        client = TestClient()

        client.save({"fmt/conanfile.py": self.conanfile.render(name="fmt", version="version")})

        client.save({"libA/conanfile.py": self.conanfile.render(name="libA", version="version",
                                                                req="fmt/version@user/channel")})
        client.save({"libB/conanfile.py": self.conanfile.render(name="libB", version="version",
                                                                req="fmt/version@user/channel",
                                                                br="libA/version@user/channel")})
        client.save({"libC/conanfile.py": self.conanfile.render(name="libC", version="version",
                                                                req="fmt/version@user/channel",
                                                                br="libB/version@user/channel")})

        client.run("create fmt fmt/version@user/channel")

        client.run("create libA libA/version@user/channel")
        client.run("create libB libB/version@user/channel")
        client.run("create libC libC/version@user/channel")

        conanws = textwrap.dedent("""
                    editables:
                      libA/version@user/channel:
                        path: libA
                      libB/version@user/channel:
                        path: libB
                      libC/version@user/channel:
                        path: libC
        
                    workspace_generator: cmake
                    root: libC/version@user/channel
                """)
        client.save({"conanws.yml": conanws})
        client.run("workspace install .")



# -*- coding: utf-8 -*-

import unittest

from parameterized import parameterized

from conans.util.files import decode_text


class DecodeTextTest(unittest.TestCase):

    @parameterized.expand([(b'\x41',),
                           (b'\xef\xbb\xbf\x41',),
                           (b'\xfe\xff\x00\x41',),
                           (b'\xff\xfe\x41\x00',),
                           (b'\x00\x00\xfe\xff\x00\x00\x00\x41',),
                           (b'\xff\xfe\x00\x00\x41\x00\x00\x00',),
                           (b'\x2b\x2f\x76\x38\x41',),
                           (b'\x2b\x2f\x76\x39\x41',),
                           (b'\x2b\x2f\x76\x2B\x41',),
                           (b'\x2b\x2f\x76\x2F\x41',),
                           (b'\x2b\x2f\x76\x38\x2d\x41',)])
    def test_audo_encodings(self, text):
        self.assertEqual('A', decode_text(text))

    @parameterized.expand([(b'\x41', "utf_8_sig"),
                           (b'\x00\x41', "utf_16_be"),
                           (b'\x41\x00', "utf_16_le"),
                           (b'\x00\x00\x00\x41', "utf_32_be"),
                           (b'\x41\x00\x00\x00', "utf_32_le"),
                           (b'\x41', "utf_7")])
    def test_explicit_encodings(self, text, encoding):
        self.assertEqual('A', decode_text(text, encoding))



import unittest

import pytest

from conans.client.tools import environment_append, six
from conans.test.utils.tools import TestClient
from conans.util.env_reader import get_env


@pytest.mark.skipif(get_env("TESTING_REVISIONS_ENABLED", False), reason="Until conan config is implemented")
@pytest.mark.skipif(six.PY2, reason="v2.0: Only testing for Python 3")
class CliHelpTest(unittest.TestCase):

    def run(self, *args, **kwargs):
        with environment_append({"CONAN_V2_CLI": "1"}):
            super(CliHelpTest, self).run(*args, **kwargs)

    def test_help_command(self):
        client = TestClient()

        client.run("help")
        self.assertIn("Shows help for a specific command", client.out)

        client.run("help search")
        self.assertIn("Searches for package recipes whose name contain", client.out)



import os
import unittest

import six

from conans.model.profile import Profile
from conans.paths import CONANFILE
from conans.test.utils.tools import  TestClient
from conans.util.files import save


conanfile_scope_env = """
from conans import ConanFile

class AConan(ConanFile):
    settings = "os"
    requires = "Hello/0.1@lasote/testing"

    def build(self):
        self.run("SET" if self.settings.os=="Windows" else "env")
"""

conanfile_dep = """
from conans import ConanFile

class AConan(ConanFile):
    name = "Hello"
    version = "0.1"

    def package_info(self):
        self.env_info.PATH=["/path/to/my/folder"]
"""


class ProfilesEnvironmentTest(unittest.TestCase):

    def setUp(self):
        self.client = TestClient()

    def test_build_with_profile(self):
        self._create_profile("scopes_env", {},
                             {"CXX": "/path/tomy/g++_build", "CC": "/path/tomy/gcc_build"})

        self.client.save({CONANFILE: conanfile_dep})
        self.client.run("export . lasote/testing")

        self.client.save({CONANFILE: conanfile_scope_env}, clean_first=True)
        self.client.run("install . --build=missing -pr scopes_env")
        self.client.run("build .")
        six.assertRegex(self, str(self.client.out), "PATH=['\"]*/path/to/my/folder")
        self._assert_env_variable_printed("CC", "/path/tomy/gcc_build")
        self._assert_env_variable_printed("CXX", "/path/tomy/g++_build")

    def _assert_env_variable_printed(self, name, value):
        self.assertIn("%s=%s" % (name, value), self.client.out)

    def _create_profile(self, name, settings, env=None):
        env = env or {}
        profile = Profile()
        profile._settings = settings or {}
        for varname, value in env.items():
            profile.env_values.add(varname, value)
        save(os.path.join(self.client.cache.profiles_path, name), "include(default)\n" + profile.dumps())



# coding=utf-8

import unittest
from types import MethodType

from parameterized import parameterized
from six import StringIO

from conans.client.output import ConanOutput, colorama_initialize
from mock import mock


class ConanOutputTest(unittest.TestCase):

    def test_blocked_output(self):
        # https://github.com/conan-io/conan/issues/4277
        stream = StringIO()

        def write_raise(self, data):
            write_raise.counter = getattr(write_raise, "counter", 0) + 1
            if write_raise.counter < 2:
                raise IOError("Stdout locked")
            self.super_write(data)
        stream.super_write = stream.write
        stream.write = MethodType(write_raise, stream)
        out = ConanOutput(stream)

        with mock.patch("time.sleep") as sleep:
            out.write("Hello world")
            sleep.assert_any_call(0.02)
        self.assertEqual("Hello world", stream.getvalue())

    @parameterized.expand([(False, {}),
                           (False, {"CONAN_COLOR_DISPLAY": "0"}),
                           (True, {"CONAN_COLOR_DISPLAY": "0"}),
                           (False, {"PYCHARM_HOSTED": "1"}),
                           (True, {"PYCHARM_HOSTED": "1", "CONAN_COLOR_DISPLAY": "0"}),
                           (True, {"NO_COLOR": ""}),
                           (True, {"CLICOLOR": "0"}),
                           (True, {"CLICOLOR": "0", "CONAN_COLOR_DISPLAY": "1"}),
                           (False, {"CLICOLOR": "1"}),
                           (False, {"CLICOLOR_FORCE": "0"}),
                           (True,
                            {"CLICOLOR": "1", "CLICOLOR_FORCE": "1", "CONAN_COLOR_DISPLAY": "1",
                             "PYCHARM_HOSTED": "1", "NO_COLOR": "1"})])
    def test_output_no_color(self, isatty, env):
        with mock.patch("colorama.init") as init:
            with mock.patch("sys.stdout.isatty", return_value=isatty), \
                 mock.patch.dict("os.environ", env, clear=True):
                assert not colorama_initialize()
                init.assert_not_called()

    @parameterized.expand([(True, {}),
                           (False, {"CONAN_COLOR_DISPLAY": "1"}),
                           (True, {"CONAN_COLOR_DISPLAY": "1"}),
                           (True, {"CLICOLOR": "1"}),
                           (True, {"CLICOLOR_FORCE": "0"})])
    def test_output_color(self, isatty, env):
        with mock.patch("colorama.init") as init:
            with mock.patch("sys.stdout.isatty", return_value=isatty), \
                 mock.patch.dict("os.environ", env, clear=True):
                assert colorama_initialize()
                init.assert_called_once_with()

    @parameterized.expand([(False, {"PYCHARM_HOSTED": "1", "CONAN_COLOR_DISPLAY": "1"}),
                           (True, {"PYCHARM_HOSTED": "1"}),
                           (False, {"CLICOLOR_FORCE": "1"}),
                           (True, {"CLICOLOR_FORCE": "1", "CLICOLOR": "0"}),
                           (True, {"CLICOLOR_FORCE": "1", "CONAN_COLOR_DISPLAY": "0"})])
    def test_output_color_prevent_strip(self, isatty, env):
        with mock.patch("colorama.init") as init:
            with mock.patch("sys.stdout.isatty", return_value=isatty), \
                 mock.patch.dict("os.environ", env, clear=True):
                assert colorama_initialize()
                init.assert_called_once_with(convert=False, strip=False)



# coding=utf-8

import os
import unittest
from collections import namedtuple

import mock

from conans.client.conf.config_installer import _process_folder
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import save_files
from conans.test.utils.mocks import TestBufferConanOutput


class ProcessFolderTests(unittest.TestCase):
    def test_config_in_empty_folder(self):
        output = TestBufferConanOutput()
        cache_t = namedtuple("_", ["cache_folder", "remotes_path"])
        config_t = namedtuple("_", ["source_folder"])

        cache_folder = temp_folder()
        remotes_json = os.path.join(cache_folder, "registry.json")
        cache = cache_t(cache_folder=cache_folder, remotes_path=remotes_json)
        ori_folder = temp_folder()
        save_files(ori_folder, {'registry.json': 'whatever'})
        with mock.patch("conans.client.conf.config_installer.migrate_registry_file",
                        return_value=None):
            self.assertFalse(os.path.exists(remotes_json))
            _process_folder(config_t(None), ori_folder, cache, output)
        self.assertTrue(os.path.exists(remotes_json))



from pathlib import Path
from conans import ConanFile
from sense2.model.layout import Folders

class Sense2File(ConanFile):

    apt_user = ""
    apt_server = ""

    def __init__(self, output, runner, display_name="", user=None, channel=None):
        super().__init__(output, runner, display_name, user, channel)
        # layout() method related variables:
        self.folders = Folders()
        
    @property
    def deb_folder(self):
        return self.folders.base_deb
    
    @property
    def deb_path(self) -> Path:
        assert self.deb_folder is not None, "`deb_folder` is `None`"
        return Path(self.deb_folder)
    
    def install_deb(self):
        pass
    
    def install_docker(self):
        pass

    def install_sense_system(self):
        pass
    
    def upload_deb(sef):
        pass



import unittest

from mock import Mock

from conans.client.generators.text import TXTGenerator
from conans.model.build_info import CppInfo, DepCppInfo
from conans.model.conan_file import ConanFile
from conans.model.env_info import EnvValues, EnvInfo
from conans.model.ref import ConanFileReference
from conans.model.settings import Settings
from conans.model.user_info import DepsUserInfo
from conans.model.user_info import UserInfo


class DumpLoadTestCase(unittest.TestCase):

    def test_names_per_generator(self):
        cpp_info = CppInfo("pkg_name", "root")
        cpp_info.name = "name"
        cpp_info.names["txt"] = "txt_name"
        cpp_info.names["cmake_find_package"] = "SpecialName"
        cpp_info.filenames["cmake_find_package"] = "SpecialFileName"
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())
        conanfile.deps_cpp_info.add("pkg_name", DepCppInfo(cpp_info))
        content = TXTGenerator(conanfile).content
        parsed_deps_cpp_info, _, _, _ = TXTGenerator.loads(content, filter_empty=False)

        parsed_cpp_info = parsed_deps_cpp_info["pkg_name"]
        self.assertEqual(parsed_cpp_info.get_name("txt"), "txt_name")
        self.assertEqual(parsed_cpp_info.get_name("cmake_find_package"), "SpecialName")
        self.assertEqual(parsed_cpp_info.get_filename("cmake_find_package"), "SpecialFileName")
        self.assertEqual(parsed_cpp_info.get_name("pkg_config"), "pkg_name")

    def test_idempotent(self):
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())

        # Add some cpp_info
        ref = ConanFileReference.loads("MyPkg/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "dummy_root_folder1")
        cpp_info.names["txt"] = "mypkg1-txt"
        cpp_info.version = ref.version
        cpp_info.defines = ["MYDEFINE1"]
        cpp_info.cxxflags = ["-cxxflag_parent"]
        cpp_info.includedirs = ["mypkg1/include"]
        cpp_info.filter_empty = False
        conanfile.deps_cpp_info.add(ref.name, cpp_info)

        ref = ConanFileReference.loads("MyPkg2/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "dummy_root_folder2")
        cpp_info.defines = ["MYDEFINE2"]
        cpp_info.cxxflags = ["-cxxflag_dep"]
        cpp_info.filter_empty = False
        conanfile.deps_cpp_info.add(ref.name, cpp_info)

        # Add env_info
        env_info = EnvInfo()
        env_info.VAR1 = "value1"
        env_info.PATH.append("path-extended")
        conanfile.deps_env_info.update(env_info, "my_pkg")

        env_info = EnvInfo()
        env_info.VAR1 = "other-value1"
        env_info.PATH.append("other-path-extended")
        conanfile.deps_env_info.update(env_info, "other-pkg")

        # Add user_info for HOST
        user_info = UserInfo()
        user_info.VAR1 = "value1"
        conanfile.deps_user_info["my_pkg"] = user_info

        user_info = UserInfo()
        user_info.VAR1 = "other-value1"
        conanfile.deps_user_info["other-pkg"] = user_info

        # Add user_info for BUILD
        conanfile.user_info_build = DepsUserInfo()
        user_info = UserInfo()
        user_info.VAR1 = "value1"
        conanfile.user_info_build["build_pkg"] = user_info

        user_info = UserInfo()
        user_info.VAR1 = "other-value1"
        conanfile.user_info_build["other-build-pkg"] = user_info

        master_content = TXTGenerator(conanfile).content
        after_cpp_info, after_user_info, after_env_info, after_user_info_build = \
            TXTGenerator.loads(master_content, filter_empty=False)
        # Assign them to a different conanfile
        other_conanfile = ConanFile(Mock(), None)
        other_conanfile.initialize(Settings({}), EnvValues())
        other_conanfile.deps_cpp_info = after_cpp_info
        other_conanfile.deps_env_info = after_env_info
        other_conanfile.deps_user_info = after_user_info
        other_conanfile.user_info_build = after_user_info_build
        after_content = TXTGenerator(other_conanfile).content

        self.assertListEqual(master_content.splitlines(), after_content.splitlines())



import os
import platform

import pytest
import textwrap

from conan.tools.cmake import CMakeToolchain
from conan.tools.microsoft.visual import vcvars_command
from ._base import BaseIntelTestCase

from conans.test.assets.sources import gen_function_cpp


cmakelists_txt = textwrap.dedent("""
    cmake_minimum_required(VERSION 2.8.12)
    project(MyApp CXX)
    find_package(hello REQUIRED)
    set(CMAKE_VERBOSE_MAKEFILE ON)
    add_executable(${CMAKE_PROJECT_NAME} main.cpp)
    set_target_properties(${CMAKE_PROJECT_NAME} PROPERTIES DEBUG_POSTFIX "d")
    target_link_libraries(${CMAKE_PROJECT_NAME} PRIVATE hello::hello)

    install(TARGETS ${CMAKE_PROJECT_NAME}
        RUNTIME DESTINATION bin
        LIBRARY DESTINATION lib
        ARCHIVE DESTINATION lib
        PUBLIC_HEADER DESTINATION include
    )

""")

conanfile_py = textwrap.dedent("""
    from conans import ConanFile
    from conan.tools.cmake import CMake, CMakeToolchain

    class App(ConanFile):
        settings = 'os', 'arch', 'compiler', 'build_type'
        exports_sources = "CMakeLists.txt", "main.cpp"
        generators = "cmake_find_package_multi"
        requires = "hello/0.1"

        _cmake = None

        def _configure_cmake(self):
            if not self._cmake:
                self._cmake = CMake(self)
                self._cmake.configure()
            return self._cmake

        def generate(self):
            tc = CMakeToolchain(self)
            tc.generate()

        def build(self):
            cmake = self._configure_cmake()
            cmake.build()

        def package(self):
            cmake = self._configure_cmake()
            cmake.install()
    """)


@pytest.mark.tool_cmake
@pytest.mark.tool_icc
@pytest.mark.xfail(reason="Intel compiler not installed yet on CI")
@pytest.mark.skipif(platform.system() != "Windows", reason="Only for windows")
class CMakeIntelTestCase(BaseIntelTestCase):

    def test_use_cmake_toolchain(self):
        self.t.save({'profile': self.profile})
        self.t.run("new hello/0.1 -s")
        self.t.run("create . hello/0.1@ -pr:h=profile")

        app = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])

        # Prepare the actual consumer package
        self.t.save({"conanfile.py": conanfile_py,
                     "main.cpp": app,
                     "CMakeLists.txt": cmakelists_txt,
                     'profile': self.profile},
                    clean_first=True)

        # Build in the cache
        self.t.run("install . -pr:h=profile")
        self.assertIn("conanfile.py: Generator cmake_find_package_multi created helloTargets.cmake",
                      self.t.out)
        self.t.run("build .")
        self.assertIn("The CXX compiler identification is Intel 19.1", self.t.out)

        exe = os.path.join("Release", "MyApp.exe")
        self.t.run_command(exe)
        self.assertIn("main __INTEL_COMPILER1910", self.t.out)

        vcvars = vcvars_command(version="15", architecture="x64")
        dumpbind_cmd = '%s && dumpbin /dependents "%s"' % (vcvars, exe)
        self.t.run_command(dumpbind_cmd)
        self.assertIn("KERNEL32.dll", self.t.out)

        # Build locally
        os.unlink(os.path.join(self.t.current_folder, exe))

        self.t.run_command('cmake . -G "Visual Studio 15 2017" '
                           '-DCMAKE_TOOLCHAIN_FILE={}'.format(CMakeToolchain.filename))
        self.t.run_command('cmake --build . --config Release')

        self.t.run_command(exe)
        self.assertIn("main __INTEL_COMPILER1910", self.t.out)

        self.t.run_command(dumpbind_cmd)
        self.assertIn("KERNEL32.dll", self.t.out)



import textwrap

from conans.test.utils.conan_v2_tests import ConanV2ModeTestCase


class ConanfileSourceTestCase(ConanV2ModeTestCase):
    """ Conan v2: 'self.info' is not available in 'package()' """

    def test_info_not_in_package(self):
        # self.info is not available in 'package'
        t = self.get_client()
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class Recipe(ConanFile):
            
                def package(self):
                    self.info.header_only()
        """)
        t.save({'conanfile.py': conanfile})
        t.run('create . name/version@ -s os=Linux', assert_error=True)
        self.assertIn("Conan v2 incompatible: 'self.info' access in package() method is deprecated", t.out)



from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


def test_env_vars_from_build_require():
    br = str(GenConanfile().with_name("hello_compiler").with_version("1.0").with_import("import os"))
    br += """
    def package_info(self):
        {}
    """
    vs = ["CC", "CC_LD", "CXX", "CXX_LD", "AR", "STRIP", "AS", "WINDRES", "PKG_CONFIG", "LD"]
    lines = "\n        ".join(['self.buildenv_info.define("{var}", "{var}_VALUE")'.format(var=var)
                               for var in vs])
    cf = br.format(lines)

    client = TestClient()
    client.save({"conanfile.py": cf})
    client.run("create .")

    conanfile = GenConanfile().with_settings("os", "arch", "compiler", "build_type")\
        .with_name("consumer").with_version("1.0").with_generator("MesonToolchain")\
        .with_build_requirement("hello_compiler/1.0")
    client.save({"conanfile.py": conanfile})
    client.run("install . -pr:h=default -pr:b=default")
    content = client.load("conan_meson_native.ini")
    assert "c = 'CC_VALUE'" in content
    assert "cpp = 'CXX_VALUE'" in content
    assert "c_ld = 'CC_LD_VALUE'" in content
    assert "cpp_ld = 'CXX_LD_VALUE'" in content
    assert "ar = 'AR_VALUE'" in content
    assert "strip = 'STRIP_VALUE'" in content
    assert "as = 'AS_VALUE'" in content
    assert "windres = 'WINDRES_VALUE'" in content
    assert "pkgconfig = 'PKG_CONFIG_VALUE'" in content

    # Now change the build require to declare only LD
    lines = '\n        self.buildenv_info.define("LD", "LD_VALUE")'
    cf = br.format(lines)
    client = TestClient()
    client.save({"conanfile.py": cf})
    client.run("create .")

    # Create the consumer again, now the LD env var will be applied
    client.save({"conanfile.py": conanfile})
    client.run("install . -pr:h=default -pr:b=default")
    content = client.load("conan_meson_native.ini")
    assert "c_ld = 'LD_VALUE'" in content
    assert "cpp_ld = 'LD_VALUE'" in content



import os
import platform
import textwrap

import pytest

from conans.test.assets.sources import gen_function_cpp, gen_function_h
from conans.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() != "Linux", reason="Only Linux")
@pytest.mark.tool_cmake
def test_shared_link_flags():
    """
    Testing CMakeDeps and linker flags injection

    Issue: https://github.com/conan-io/conan/issues/9936
    """
    conanfile = textwrap.dedent("""
    from conans import ConanFile
    from conan.tools.cmake import CMake, cmake_layout


    class HelloConan(ConanFile):
        name = "hello"
        version = "1.0"
        settings = "os", "compiler", "build_type", "arch"
        options = {"shared": [True, False]}
        default_options = {"shared": False}
        exports_sources = "CMakeLists.txt", "src/*", "include/*"
        generators = "CMakeDeps", "CMakeToolchain"

        def layout(self):
            cmake_layout(self)

        def build(self):
            cmake = CMake(self)
            cmake.configure()
            cmake.build()

        def package(self):
            cmake = CMake(self)
            cmake.install()

        def package_info(self):
            self.cpp_info.libs = ["hello"]
            self.cpp_info.sharedlinkflags = ["-z now", "-z relro"]
            self.cpp_info.exelinkflags = ["-z now", "-z relro"]
    """)

    client = TestClient()
    client.run("new hello/1.0 -m cmake_lib")
    client.save({"conanfile.py": conanfile})
    client.run("create .")
    t = os.path.join("test_package", "build", "generators", "hello-release-x86_64-data.cmake")
    target_data_cmake_content = client.load(t)
    assert 'set(hello_SHARED_LINK_FLAGS_RELEASE "-z now;-z relro")' in target_data_cmake_content
    assert 'set(hello_EXE_LINK_FLAGS_RELEASE "-z now;-z relro")' in target_data_cmake_content
    assert "hello/1.0: Hello World Release!" in client.out


def test_not_mixed_configurations():
    # https://github.com/conan-io/conan/issues/11852

    client = TestClient()
    client.run("new foo/1.0 -m cmake_lib")

    conanfile = client.load("conanfile.py")
    conanfile.replace("package_info(self)", "invalid(self)")
    conanfile += """
    def package_info(self):
        self.cpp_info.libs = ["foo" if self.settings.build_type == "Release" else "foo_d"]
    """

    cmake = textwrap.dedent("""
            cmake_minimum_required(VERSION 3.15)
            project(foo CXX)

            add_library(foo "src/foo.cpp")
            target_include_directories(foo PUBLIC include)
            set_target_properties(foo PROPERTIES PUBLIC_HEADER "include/foo.h")

            # Different name for Release or Debug
            set_target_properties(foo PROPERTIES OUTPUT_NAME_DEBUG foo_d)
            set_target_properties(foo PROPERTIES OUTPUT_NAME_RELEASE foo)
            install(TARGETS foo)
    """)

    client.save({"CMakeLists.txt": cmake, "conanfile.py": conanfile})
    client.run("create .")
    client.run("create . -s build_type=Debug")
    if platform.system() != "Windows":
        assert "libfoo_d.a" in client.out  # Just to make sure we built the foo_d

    # Now create a consumer of foo with CMakeDeps locally
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import CMakeToolchain, CMake, cmake_layout

        class ConsumerConan(ConanFile):
            name = "consumer"
            version = "1.0"
            settings = "os", "compiler", "build_type", "arch"
            requires = "foo/1.0"
            generators = "CMakeDeps"

            def layout(self):
                cmake_layout(self)

            def generate(self):
                tc = CMakeToolchain(self)
                tc.cache_variables["CMAKE_VERBOSE_MAKEFILE:BOOL"] = "ON"
                tc.generate()

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()

            """)
    cmake = textwrap.dedent("""
            cmake_minimum_required(VERSION 3.15)
            project(consumer CXX)

            find_package(foo)

            add_executable(consumer src/consumer.cpp)
            target_include_directories(consumer PUBLIC include)
            target_link_libraries(consumer foo::foo)

            set_target_properties(consumer PROPERTIES PUBLIC_HEADER "include/consumer.h")""")
    consumer_cpp = gen_function_cpp(name="main", includes=["foo"], calls=["foo"])
    consumer_h = gen_function_h(name="consumer")

    client.save({"conanfile.py": conanfile, "CMakeLists.txt": cmake,
                 "src/consumer.cpp": consumer_cpp, "src/consumer.h": consumer_h})

    client.run("install . -s build_type=Debug")
    client.run("install . -s build_type=Release")
    # With the bug, this build only fail on windows
    client.run("build .")

    # But we inspect the output for Macos/Linux to check that the library is not linked
    if platform.system() != "Windows":
        assert "libfoo_d.a" not in client.out
        assert "libfoo.a" in client.out



import mock
import os
import unittest

from conans import Settings
from conans.client.conf import get_default_settings_yml
from conans.client.tools.env import environment_append
from conans.client.tools.intel import intel_compilervars_command
from conans.errors import ConanException
from conans.test.utils.mocks import MockConanfile


class CompilerVarsTest(unittest.TestCase):
    def test_already_set(self):
        with environment_append({"PSTLROOT": "1"}):
            settings = Settings.loads(get_default_settings_yml())
            cvars = intel_compilervars_command(MockConanfile(settings))
            self.assertEqual("echo Conan:intel_compilervars already set", cvars)

    def test_bas_os(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "SunOS"
        with self.assertRaises(ConanException):
            intel_compilervars_command(MockConanfile(settings))

    def test_win(self):
        install_dir = "C:\\Intel"
        with mock.patch("platform.system", mock.MagicMock(return_value="Windows")),\
            mock.patch("conans.client.tools.intel.intel_installation_path",
                       mock.MagicMock(return_value=install_dir)):
            settings = Settings.loads(get_default_settings_yml())
            settings.os = "Windows"
            settings.compiler = "intel"
            settings.compiler.base = "Visual Studio"
            settings.arch = "ppc32"
            with self.assertRaises(ConanException):
                intel_compilervars_command(MockConanfile(settings))

            path = os.path.join(install_dir, "bin", "compilervars.bat")

            settings.arch = "x86"
            cvars = intel_compilervars_command(MockConanfile(settings))
            expected = 'call "%s" -arch ia32' % path
            self.assertEqual(expected, cvars)

            settings.compiler.base.version = "16"
            cvars = intel_compilervars_command(MockConanfile(settings))
            expected = 'call "%s" -arch ia32 vs2019' % path
            self.assertEqual(expected, cvars)

            settings.arch = "x86_64"
            expected = 'call "%s" -arch intel64 vs2019' % path
            cvars = intel_compilervars_command(MockConanfile(settings))
            self.assertEqual(expected, cvars)

    def test_linux(self):
        install_dir = "/opt/intel"
        with mock.patch("platform.system", mock.MagicMock(return_value="Linux")),\
            mock.patch("conans.client.tools.intel.intel_installation_path",
                       mock.MagicMock(return_value="/opt/intel")):
            settings = Settings.loads(get_default_settings_yml())
            settings.os = "Linux"
            settings.compiler = "intel"
            settings.compiler.base = "gcc"
            settings.arch = "ppc32"
            with self.assertRaises(ConanException):
                intel_compilervars_command(MockConanfile(settings))

            path = os.path.join(install_dir, "bin", "compilervars.sh")

            settings.arch = "x86"
            cvars = intel_compilervars_command(MockConanfile(settings))
            expected = 'COMPILERVARS_PLATFORM=linux COMPILERVARS_ARCHITECTURE=ia32 . ' \
                       '"%s" -arch ia32 -platform linux' % path
            self.assertEqual(expected, cvars)

            settings.arch = "x86_64"
            expected = 'COMPILERVARS_PLATFORM=linux COMPILERVARS_ARCHITECTURE=intel64 . ' \
                       '"%s" -arch intel64 -platform linux' % path
            cvars = intel_compilervars_command(MockConanfile(settings))
            self.assertEqual(expected, cvars)

    def test_mac(self):
        install_dir = "/opt/intel"
        with mock.patch("platform.system", mock.MagicMock(return_value="Darwin")),\
            mock.patch("conans.client.tools.intel.intel_installation_path",
                       mock.MagicMock(return_value="/opt/intel")):
            settings = Settings.loads(get_default_settings_yml())
            settings.os = "Macos"
            settings.compiler = "intel"
            settings.compiler.base = "apple-clang"
            settings.arch = "ppc32"
            with self.assertRaises(ConanException):
                intel_compilervars_command(MockConanfile(settings))

            path = os.path.join(install_dir, "bin", "compilervars.sh")

            settings.arch = "x86"
            cvars = intel_compilervars_command(MockConanfile(settings))
            expected = 'COMPILERVARS_PLATFORM=mac COMPILERVARS_ARCHITECTURE=ia32 . ' \
                       '"%s" -arch ia32 -platform mac' % path
            self.assertEqual(expected, cvars)

            settings.arch = "x86_64"
            expected = 'COMPILERVARS_PLATFORM=mac COMPILERVARS_ARCHITECTURE=intel64 . ' \
                       '"%s" -arch intel64 -platform mac' % path
            cvars = intel_compilervars_command(MockConanfile(settings))
            self.assertEqual(expected, cvars)



import os

import patch_ng
import pytest

from conan.tools.files import patch, apply_conandata_patches
from conans.errors import ConanException
from conans.test.utils.mocks import ConanFileMock


class MockPatchset:
    filename = None
    string = None
    apply_args = None

    def apply(self, root, strip, fuzz):
        self.apply_args = (root, strip, fuzz)
        return True


@pytest.fixture
def mock_patch_ng(monkeypatch):
    mock = MockPatchset()

    def mock_fromfile(filename):
        mock.filename = filename
        return mock

    def mock_fromstring(string):
        mock.string = string
        return mock

    monkeypatch.setattr(patch_ng, "fromfile", mock_fromfile)
    monkeypatch.setattr(patch_ng, "fromstring", mock_fromstring)
    return mock


def test_single_patch_file(mock_patch_ng):
    conanfile = ConanFileMock()
    conanfile.folders.set_base_source("/my_source")
    conanfile.folders.set_base_export_sources("/my_source")
    conanfile.display_name = 'mocked/ref'
    patch(conanfile, patch_file='patch-file')
    assert mock_patch_ng.filename.replace("\\", "/") == '/my_source/patch-file'
    assert mock_patch_ng.string is None
    assert mock_patch_ng.apply_args == ("/my_source", 0, False)
    assert len(str(conanfile.output)) == 0


def test_single_patch_file_from_forced_build(mock_patch_ng):
    conanfile = ConanFileMock()
    conanfile.folders.set_base_source("/my_source")
    conanfile.folders.set_base_export_sources("/my_source")
    conanfile.display_name = 'mocked/ref'
    patch(conanfile, patch_file='/my_build/patch-file')
    assert mock_patch_ng.filename == '/my_build/patch-file'
    assert mock_patch_ng.string is None
    assert mock_patch_ng.apply_args == ("/my_source", 0, False)
    assert len(str(conanfile.output)) == 0


def test_base_path(mock_patch_ng):
    conanfile = ConanFileMock()
    conanfile.folders.set_base_source("/my_source")
    conanfile.folders.set_base_export_sources("/my_source")
    conanfile.folders.source = "src"  # This not applies to find the patch file but for applying it
    conanfile.display_name = 'mocked/ref'
    patch(conanfile, patch_file='patch-file', base_path="subfolder")
    assert mock_patch_ng.filename.replace("\\", "/") == '/my_source/patch-file'
    assert mock_patch_ng.string is None
    assert mock_patch_ng.apply_args == (os.path.join("/my_source", "src", "subfolder"), 0, False)
    assert len(str(conanfile.output)) == 0


def test_apply_in_build_from_patch_in_source(mock_patch_ng):
    conanfile = ConanFileMock()
    conanfile.folders.set_base_source("/my_source")
    conanfile.folders.set_base_export_sources("/my_source")
    conanfile.display_name = 'mocked/ref'
    patch(conanfile, patch_file='patch-file', base_path="/my_build/subfolder")
    assert mock_patch_ng.filename.replace("\\", "/") == '/my_source/patch-file'
    assert mock_patch_ng.string is None
    assert mock_patch_ng.apply_args[0] == os.path.join("/my_build", "subfolder").replace("\\", "/")
    assert mock_patch_ng.apply_args[1] == 0
    assert mock_patch_ng.apply_args[2] is False
    assert len(str(conanfile.output)) == 0


def test_single_patch_string(mock_patch_ng):
    conanfile = ConanFileMock()
    conanfile.folders.set_base_source("my_folder")
    conanfile.folders.set_base_export_sources("my_folder")
    conanfile.display_name = 'mocked/ref'
    patch(conanfile, patch_string='patch_string')
    assert mock_patch_ng.string == b'patch_string'
    assert mock_patch_ng.filename is None
    assert mock_patch_ng.apply_args == ("my_folder", 0, False)
    assert len(str(conanfile.output)) == 0


def test_single_patch_arguments(mock_patch_ng):
    conanfile = ConanFileMock()
    conanfile.display_name = 'mocked/ref'
    conanfile.folders.set_base_source("/path/to/sources")
    conanfile.folders.set_base_export_sources("/path/to/sources")
    patch(conanfile, patch_file='patch-file', strip=23, fuzz=True)
    assert mock_patch_ng.filename.replace("\\", "/") == '/path/to/sources/patch-file'
    assert mock_patch_ng.apply_args == ("/path/to/sources", 23, True)
    assert len(str(conanfile.output)) == 0


def test_single_patch_type(mock_patch_ng):
    conanfile = ConanFileMock()
    conanfile.display_name = 'mocked/ref'
    patch(conanfile, patch_file='patch-file', patch_type='patch_type')
    assert 'Apply patch (patch_type)\n' == str(conanfile.output)


def test_single_patch_description(mock_patch_ng):
    conanfile = ConanFileMock()
    conanfile.display_name = 'mocked/ref'
    patch(conanfile, patch_file='patch-file', patch_description='patch_description')
    assert 'Apply patch: patch_description\n' == str(conanfile.output)


def test_single_patch_extra_fields(mock_patch_ng):
    conanfile = ConanFileMock()
    conanfile.display_name = 'mocked/ref'
    patch(conanfile, patch_file='patch-file', patch_type='patch_type',
          patch_description='patch_description')
    assert 'Apply patch (patch_type): patch_description\n' == str(conanfile.output)


def test_single_no_patchset(monkeypatch):
    monkeypatch.setattr(patch_ng, "fromfile", lambda _: None)

    conanfile = ConanFileMock()
    conanfile.display_name = 'mocked/ref'
    with pytest.raises(ConanException) as excinfo:
        patch(conanfile, patch_file='patch-file-failed')
    assert 'Failed to parse patch: patch-file-failed' == str(excinfo.value)


def test_single_apply_fail(monkeypatch):
    class MockedApply:
        def apply(self, *args, **kwargs):
            return False

    monkeypatch.setattr(patch_ng, "fromfile", lambda _: MockedApply())

    conanfile = ConanFileMock()
    conanfile.display_name = 'mocked/ref'
    with pytest.raises(ConanException) as excinfo:
        patch(conanfile, patch_file='patch-file-failed')
    assert 'Failed to apply patch: patch-file-failed' == str(excinfo.value)


def test_multiple_no_version(mock_patch_ng):
    conanfile = ConanFileMock()
    conanfile.display_name = 'mocked/ref'
    conanfile.conan_data = {'patches': [
        {'patch_file': 'patches/0001-buildflatbuffers-cmake.patch',
         'base_path': 'source_subfolder', },
        {'patch_file': 'patches/0002-implicit-copy-constructor.patch',
         'base_path': 'source_subfolder',
         'patch_type': 'backport',
         'patch_source': 'https://github.com/google/flatbuffers/pull/5650',
         'patch_description': 'Needed to build with modern clang compilers.'}
    ]}
    apply_conandata_patches(conanfile)
    assert 'Apply patch (backport): Needed to build with modern clang compilers.\n' \
           == str(conanfile.output)


def test_multiple_with_version(mock_patch_ng):
    conanfile = ConanFileMock()
    conanfile.display_name = 'mocked/ref'
    conandata_contents = {'patches': {
        "1.11.0": [
            {'patch_file': 'patches/0001-buildflatbuffers-cmake.patch',
             'base_path': 'source_subfolder', },
            {'patch_file': 'patches/0002-implicit-copy-constructor.patch',
             'base_path': 'source_subfolder',
             'patch_type': 'backport',
             'patch_source': 'https://github.com/google/flatbuffers/pull/5650',
             'patch_description': 'Needed to build with modern clang compilers.'}
        ],
        "1.12.0": [
            {'patch_file': 'patches/0001-buildflatbuffers-cmake.patch',
             'base_path': 'source_subfolder', },
        ]}}

    conanfile.conan_data = conandata_contents.copy()

    with pytest.raises(AssertionError) as excinfo:
        apply_conandata_patches(conanfile)
    assert 'Can only be applied if conanfile.version is already defined' == str(excinfo.value)

    conanfile.version = "1.2.11"
    apply_conandata_patches(conanfile)
    assert len(str(conanfile.output)) == 0

    conanfile.version = "1.11.0"
    apply_conandata_patches(conanfile)
    assert 'Apply patch (backport): Needed to build with modern clang compilers.\n' \
           == str(conanfile.output)
    
    # Ensure the function is not mutating the `conan_data` structure
    assert conanfile.conan_data == conandata_contents



import os

import pytest

from conans.model.manifest import FileTreeManifest
from conans.model.ref import ConanFileReference
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


@pytest.mark.parametrize("relative_path", [False, True])
def test_basic(relative_path):
    client = TestClient()
    source_folder = os.path.join(client.current_folder, "source")
    files = {"conanfile.py": GenConanfile().with_exports("*"),
             "main.cpp": "mymain"}
    client.save(files, path=source_folder)
    if relative_path:
        with client.chdir("current"):
            client.run("export ../source hello/0.1@lasote/stable")
    else:
        client.run("export source hello/0.1@lasote/stable")

    # The result should be the same in both cases
    ref = ConanFileReference("hello", "0.1", "lasote", "stable")
    reg_path = client.cache.package_layout(ref).export()
    manif = FileTreeManifest.load(reg_path)

    assert '%s: A new conanfile.py version was exported' % str(ref) in client.out
    assert '%s: Folder: %s' % (str(ref), reg_path) in client.out

    for name in list(files.keys()):
        assert os.path.exists(os.path.join(reg_path, name))

    expected_sums = {'conanfile.py': '1b7c687fffb8544bd2de497cd7a6eee6',
                     'main.cpp': '76c0a7a9d385266e27d69d3875f6ac19'}
    assert expected_sums == manif.file_sums


@pytest.mark.parametrize("relative_path", [False, True])
def test_path(relative_path):
    client = TestClient()
    if relative_path:
        client.save({"conan/conanfile.py": GenConanfile().with_exports("../source*"),
                     "source/main.cpp": "mymain"})
        with client.chdir("current"):
            client.run("export ../conan hello/0.1@lasote/stable")
    else:
        client.save({"current/conanfile.py": GenConanfile().with_exports("../source*"),
                     "source/main.cpp": "mymain"})
        with client.chdir("current"):
            client.run("export . hello/0.1@lasote/stable")

    ref = ConanFileReference("hello", "0.1", "lasote", "stable")
    reg_path = client.cache.package_layout(ref).export()
    manif = FileTreeManifest.load(reg_path)

    for name in ['conanfile.py', 'conanmanifest.txt', 'source/main.cpp']:
        assert os.path.exists(os.path.join(reg_path, name))

    expected_sums = {'conanfile.py': '718d94ef217b17297e10d60f1132ccf5',
                     'source/main.cpp': '76c0a7a9d385266e27d69d3875f6ac19'}
    assert expected_sums == manif.file_sums
